#!/usr/bin/env python
"""Original NetworkX graph tests"""
import pytest
import networkx
import networkx as nx
from networkx.testing.utils import *

from .historical_tests import HistoricalTests


class TestDiGraphHistorical(HistoricalTests):

    @classmethod
    def setup_class(cls):
        HistoricalTests.setup_class()
        cls.G = nx.DiGraph

    def test_in_degree(self):
        G = self.G()
        G.add_nodes_from('GJK')
        G.add_edges_from([('A', 'B'), ('A', 'C'), ('B', 'D'),
                          ('B', 'C'), ('C', 'D')])

        assert sorted(d for n, d in G.in_degree()) == [0, 0, 0, 0, 1, 2, 2]
        assert (dict(G.in_degree()) ==
                     {'A': 0, 'C': 2, 'B': 1, 'D': 2, 'G': 0, 'K': 0, 'J': 0})

    def test_out_degree(self):
        G = self.G()
        G.add_nodes_from('GJK')
        G.add_edges_from([('A', 'B'), ('A', 'C'), ('B', 'D'),
                          ('B', 'C'), ('C', 'D')])
        assert (sorted([v for k, v in G.in_degree()]) ==
                     [0, 0, 0, 0, 1, 2, 2])
        assert (dict(G.out_degree()) ==
                     {'A': 2, 'C': 1, 'B': 2, 'D': 0, 'G': 0, 'K': 0, 'J': 0})

    def test_degree_digraph(self):
        H = nx.DiGraph()
        H.add_edges_from([(1, 24), (1, 2)])
        assert sorted(d for n, d in H.in_degree([1, 24])) == [0, 1]
        assert sorted(d for n, d in H.out_degree([1, 24])) == [0, 2]
        assert sorted(d for n, d in H.degree([1, 24])) == [1, 2]

    def test_neighbors(self):
        G = self.G()
        G.add_nodes_from('GJK')
        G.add_edges_from([('A', 'B'), ('A', 'C'), ('B', 'D'),
                          ('B', 'C'), ('C', 'D')])

        assert sorted(G.neighbors('C')) == ['D']
        assert sorted(G['C']) == ['D']
        assert sorted(G.neighbors('A')) == ['B', 'C']
        pytest.raises(nx.NetworkXError, G.neighbors, 'j')
        pytest.raises(nx.NetworkXError, G.neighbors, 'j')

    def test_successors(self):
        G = self.G()
        G.add_nodes_from('GJK')
        G.add_edges_from([('A', 'B'), ('A', 'C'), ('B', 'D'),
                          ('B', 'C'), ('C', 'D')])
        assert sorted(G.successors('A')) == ['B', 'C']
        assert sorted(G.successors('A')) == ['B', 'C']
        assert sorted(G.successors('G')) == []
        assert sorted(G.successors('D')) == []
        assert sorted(G.successors('G')) == []
        pytest.raises(nx.NetworkXError, G.successors, 'j')
        pytest.raises(nx.NetworkXError, G.successors, 'j')

    def test_predecessors(self):
        G = self.G()
        G.add_nodes_from('GJK')
        G.add_edges_from([('A', 'B'), ('A', 'C'), ('B', 'D'),
                          ('B', 'C'), ('C', 'D')])
        assert sorted(G.predecessors('C')) == ['A', 'B']
        assert sorted(G.predecessors('C')) == ['A', 'B']
        assert sorted(G.predecessors('G')) == []
        assert sorted(G.predecessors('A')) == []
        assert sorted(G.predecessors('G')) == []
        assert sorted(G.predecessors('A')) == []
        assert sorted(G.successors('D')) == []

        pytest.raises(nx.NetworkXError, G.predecessors, 'j')
        pytest.raises(nx.NetworkXError, G.predecessors, 'j')

    def test_reverse(self):
        G = nx.complete_graph(10)
        H = G.to_directed()
        HR = H.reverse()
        assert nx.is_isomorphic(H, HR)
        assert sorted(H.edges()) == sorted(HR.edges())

    def test_reverse2(self):
        H = nx.DiGraph()
        foo = [H.add_edge(u, u + 1) for u in range(0, 5)]
        HR = H.reverse()
        for u in range(0, 5):
            assert HR.has_edge(u + 1, u)

    def test_reverse3(self):
        H = nx.DiGraph()
        H.add_nodes_from([1, 2, 3, 4])
        HR = H.reverse()
        assert sorted(HR.nodes()) == [1, 2, 3, 4]



#!/usr/bin/env python
import pytest
import networkx as nx
from networkx import convert_node_labels_to_integers as cnlti
from networkx import NetworkXNotImplemented


class TestConnected:

    @classmethod
    def setup_class(cls):
        G1 = cnlti(nx.grid_2d_graph(2, 2), first_label=0, ordering="sorted")
        G2 = cnlti(nx.lollipop_graph(3, 3), first_label=4, ordering="sorted")
        G3 = cnlti(nx.house_graph(), first_label=10, ordering="sorted")
        cls.G = nx.union(G1, G2)
        cls.G = nx.union(cls.G, G3)
        cls.DG = nx.DiGraph([(1, 2), (1, 3), (2, 3)])
        cls.grid = cnlti(nx.grid_2d_graph(4, 4), first_label=1)

        cls.gc = []
        G = nx.DiGraph()
        G.add_edges_from([(1, 2), (2, 3), (2, 8), (3, 4), (3, 7), (4, 5),
                          (5, 3), (5, 6), (7, 4), (7, 6), (8, 1), (8, 7)])
        C = [[3, 4, 5, 7], [1, 2, 8], [6]]
        cls.gc.append((G, C))

        G = nx.DiGraph()
        G.add_edges_from([(1, 2), (1, 3), (1, 4), (4, 2), (3, 4), (2, 3)])
        C = [[2, 3, 4], [1]]
        cls.gc.append((G, C))

        G = nx.DiGraph()
        G.add_edges_from([(1, 2), (2, 3), (3, 2), (2, 1)])
        C = [[1, 2, 3]]
        cls.gc.append((G, C))

        # Eppstein's tests
        G = nx.DiGraph({0: [1], 1: [2, 3], 2: [4, 5], 3: [4, 5], 4: [6], 5: [], 6: []})
        C = [[0], [1], [2], [3], [4], [5], [6]]
        cls.gc.append((G, C))

        G = nx.DiGraph({0: [1], 1: [2, 3, 4], 2: [0, 3], 3: [4], 4: [3]})
        C = [[0, 1, 2], [3, 4]]
        cls.gc.append((G, C))

        G = nx.DiGraph()
        C = []
        cls.gc.append((G, C))

    def test_connected_components(self):
        cc = nx.connected_components
        G = self.G
        C = {
            frozenset([0, 1, 2, 3]),
            frozenset([4, 5, 6, 7, 8, 9]),
            frozenset([10, 11, 12, 13, 14])
        }
        assert {frozenset(g) for g in cc(G)} == C

    def test_number_connected_components(self):
        ncc = nx.number_connected_components
        assert ncc(self.G) == 3

    def test_number_connected_components2(self):
        ncc = nx.number_connected_components
        assert ncc(self.grid) == 1

    def test_connected_components2(self):
        cc = nx.connected_components
        G = self.grid
        C = {frozenset([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16])}
        assert {frozenset(g) for g in cc(G)} == C

    def test_node_connected_components(self):
        ncc = nx.node_connected_component
        G = self.grid
        C = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16}
        assert ncc(G, 1) == C

    def test_is_connected(self):
        assert nx.is_connected(self.grid)
        G = nx.Graph()
        G.add_nodes_from([1, 2])
        assert not nx.is_connected(G)

    def test_connected_raise(self):
        pytest.raises(NetworkXNotImplemented, nx.connected_components, self.DG)
        pytest.raises(NetworkXNotImplemented, nx.number_connected_components, self.DG)
        pytest.raises(NetworkXNotImplemented, nx.node_connected_component, self.DG, 1)
        pytest.raises(NetworkXNotImplemented, nx.is_connected, self.DG)
        pytest.raises(nx.NetworkXPointlessConcept, nx.is_connected, nx.Graph())



"""
Famous social networks.
"""
import networkx as nx
__author__ = """\n""".join(['Jordi Torrents <jtorrents@milnou.net>',
                            'Katy Bold <kbold@princeton.edu>',
                            'Aric Hagberg <aric.hagberg@gmail.com)',
                            'Issa Annamoradnejad <i.moradnejad@gmail.com>'])

__all__ = ['karate_club_graph', 'davis_southern_women_graph',
           'florentine_families_graph', 'les_miserables_graph']


def karate_club_graph():
    """Returns Zachary's Karate Club graph.

    Each node in the returned graph has a node attribute 'club' that
    indicates the name of the club to which the member represented by that node
    belongs, either 'Mr. Hi' or 'Officer'.

    Examples
    --------
    To get the name of the club to which a node belongs::

        >>> import networkx as nx
        >>> G = nx.karate_club_graph()
        >>> G.nodes[5]['club']
        'Mr. Hi'
        >>> G.nodes[9]['club']
        'Officer'

    References
    ----------
    .. [1] Zachary, Wayne W.
       "An Information Flow Model for Conflict and Fission in Small Groups."
       *Journal of Anthropological Research*, 33, 452--473, (1977).

    .. [2] Data file from:
       http://vlado.fmf.uni-lj.si/pub/networks/data/Ucinet/UciData.htm
    """
    # Create the set of all members, and the members of each club.
    all_members = set(range(34))
    club1 = {0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 12, 13, 16, 17, 19, 21}
    # club2 = all_members - club1

    G = nx.Graph()
    G.add_nodes_from(all_members)
    G.name = "Zachary's Karate Club"

    zacharydat = """\
0 1 1 1 1 1 1 1 1 0 1 1 1 1 0 0 0 1 0 1 0 1 0 0 0 0 0 0 0 0 0 1 0 0
1 0 1 1 0 0 0 1 0 0 0 0 0 1 0 0 0 1 0 1 0 1 0 0 0 0 0 0 0 0 1 0 0 0
1 1 0 1 0 0 0 1 1 1 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 1 0
1 1 1 0 0 0 0 1 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
1 0 0 0 0 0 1 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
1 0 0 0 0 0 1 0 0 0 1 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
1 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
1 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 1
0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1
1 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1
0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1
0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1
0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1
1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1
0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1
1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1
0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 1 0 0 1 1
0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 0 0 1 0 0
0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 1 0 0
0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1
0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 1
0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1
0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 1 0 0 0 0 0 1 1
0 1 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1
1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 1 0 0 0 1 1
0 0 1 0 0 0 0 0 1 0 0 0 0 0 1 1 0 0 1 0 1 0 1 1 0 0 0 0 0 1 1 1 0 1
0 0 0 0 0 0 0 0 1 1 0 0 0 1 1 1 0 0 1 1 1 0 1 1 0 0 1 1 1 1 1 1 1 0"""

    for row, line in enumerate(zacharydat.split('\n')):
        thisrow = [int(b) for b in line.split()]
        for col, entry in enumerate(thisrow):
            if entry == 1:
                G.add_edge(row, col)

    # Add the name of each member's club as a node attribute.
    for v in G:
        G.nodes[v]['club'] = 'Mr. Hi' if v in club1 else 'Officer'
    return G


def davis_southern_women_graph():
    """Returns Davis Southern women social network.

    This is a bipartite graph.

    References
    ----------
    .. [1] A. Davis, Gardner, B. B., Gardner, M. R., 1941. Deep South.
        University of Chicago Press, Chicago, IL.
    """
    G = nx.Graph()
    # Top nodes
    women = ["Evelyn Jefferson",
             "Laura Mandeville",
             "Theresa Anderson",
             "Brenda Rogers",
             "Charlotte McDowd",
             "Frances Anderson",
             "Eleanor Nye",
             "Pearl Oglethorpe",
             "Ruth DeSand",
             "Verne Sanderson",
             "Myra Liddel",
             "Katherina Rogers",
             "Sylvia Avondale",
             "Nora Fayette",
             "Helen Lloyd",
             "Dorothy Murchison",
             "Olivia Carleton",
             "Flora Price"]
    G.add_nodes_from(women, bipartite=0)
    # Bottom nodes
    events = ["E1",
              "E2",
              "E3",
              "E4",
              "E5",
              "E6",
              "E7",
              "E8",
              "E9",
              "E10",
              "E11",
              "E12",
              "E13",
              "E14"]
    G.add_nodes_from(events, bipartite=1)

    G.add_edges_from([("Evelyn Jefferson", "E1"),
                      ("Evelyn Jefferson", "E2"),
                      ("Evelyn Jefferson", "E3"),
                      ("Evelyn Jefferson", "E4"),
                      ("Evelyn Jefferson", "E5"),
                      ("Evelyn Jefferson", "E6"),
                      ("Evelyn Jefferson", "E8"),
                      ("Evelyn Jefferson", "E9"),
                      ("Laura Mandeville", "E1"),
                      ("Laura Mandeville", "E2"),
                      ("Laura Mandeville", "E3"),
                      ("Laura Mandeville", "E5"),
                      ("Laura Mandeville", "E6"),
                      ("Laura Mandeville", "E7"),
                      ("Laura Mandeville", "E8"),
                      ("Theresa Anderson", "E2"),
                      ("Theresa Anderson", "E3"),
                      ("Theresa Anderson", "E4"),
                      ("Theresa Anderson", "E5"),
                      ("Theresa Anderson", "E6"),
                      ("Theresa Anderson", "E7"),
                      ("Theresa Anderson", "E8"),
                      ("Theresa Anderson", "E9"),
                      ("Brenda Rogers", "E1"),
                      ("Brenda Rogers", "E3"),
                      ("Brenda Rogers", "E4"),
                      ("Brenda Rogers", "E5"),
                      ("Brenda Rogers", "E6"),
                      ("Brenda Rogers", "E7"),
                      ("Brenda Rogers", "E8"),
                      ("Charlotte McDowd", "E3"),
                      ("Charlotte McDowd", "E4"),
                      ("Charlotte McDowd", "E5"),
                      ("Charlotte McDowd", "E7"),
                      ("Frances Anderson", "E3"),
                      ("Frances Anderson", "E5"),
                      ("Frances Anderson", "E6"),
                      ("Frances Anderson", "E8"),
                      ("Eleanor Nye", "E5"),
                      ("Eleanor Nye", "E6"),
                      ("Eleanor Nye", "E7"),
                      ("Eleanor Nye", "E8"),
                      ("Pearl Oglethorpe", "E6"),
                      ("Pearl Oglethorpe", "E8"),
                      ("Pearl Oglethorpe", "E9"),
                      ("Ruth DeSand", "E5"),
                      ("Ruth DeSand", "E7"),
                      ("Ruth DeSand", "E8"),
                      ("Ruth DeSand", "E9"),
                      ("Verne Sanderson", "E7"),
                      ("Verne Sanderson", "E8"),
                      ("Verne Sanderson", "E9"),
                      ("Verne Sanderson", "E12"),
                      ("Myra Liddel", "E8"),
                      ("Myra Liddel", "E9"),
                      ("Myra Liddel", "E10"),
                      ("Myra Liddel", "E12"),
                      ("Katherina Rogers", "E8"),
                      ("Katherina Rogers", "E9"),
                      ("Katherina Rogers", "E10"),
                      ("Katherina Rogers", "E12"),
                      ("Katherina Rogers", "E13"),
                      ("Katherina Rogers", "E14"),
                      ("Sylvia Avondale", "E7"),
                      ("Sylvia Avondale", "E8"),
                      ("Sylvia Avondale", "E9"),
                      ("Sylvia Avondale", "E10"),
                      ("Sylvia Avondale", "E12"),
                      ("Sylvia Avondale", "E13"),
                      ("Sylvia Avondale", "E14"),
                      ("Nora Fayette", "E6"),
                      ("Nora Fayette", "E7"),
                      ("Nora Fayette", "E9"),
                      ("Nora Fayette", "E10"),
                      ("Nora Fayette", "E11"),
                      ("Nora Fayette", "E12"),
                      ("Nora Fayette", "E13"),
                      ("Nora Fayette", "E14"),
                      ("Helen Lloyd", "E7"),
                      ("Helen Lloyd", "E8"),
                      ("Helen Lloyd", "E10"),
                      ("Helen Lloyd", "E11"),
                      ("Helen Lloyd", "E12"),
                      ("Dorothy Murchison", "E8"),
                      ("Dorothy Murchison", "E9"),
                      ("Olivia Carleton", "E9"),
                      ("Olivia Carleton", "E11"),
                      ("Flora Price", "E9"),
                      ("Flora Price", "E11")])
    G.graph['top'] = women
    G.graph['bottom'] = events
    return G


def florentine_families_graph():
    """Returns Florentine families graph.

    References
    ----------
    .. [1] Ronald L. Breiger and Philippa E. Pattison
       Cumulated social roles: The duality of persons and their algebras,1
       Social Networks, Volume 8, Issue 3, September 1986, Pages 215-256
    """
    G = nx.Graph()
    G.add_edge('Acciaiuoli', 'Medici')
    G.add_edge('Castellani', 'Peruzzi')
    G.add_edge('Castellani', 'Strozzi')
    G.add_edge('Castellani', 'Barbadori')
    G.add_edge('Medici', 'Barbadori')
    G.add_edge('Medici', 'Ridolfi')
    G.add_edge('Medici', 'Tornabuoni')
    G.add_edge('Medici', 'Albizzi')
    G.add_edge('Medici', 'Salviati')
    G.add_edge('Salviati', 'Pazzi')
    G.add_edge('Peruzzi', 'Strozzi')
    G.add_edge('Peruzzi', 'Bischeri')
    G.add_edge('Strozzi', 'Ridolfi')
    G.add_edge('Strozzi', 'Bischeri')
    G.add_edge('Ridolfi', 'Tornabuoni')
    G.add_edge('Tornabuoni', 'Guadagni')
    G.add_edge('Albizzi', 'Ginori')
    G.add_edge('Albizzi', 'Guadagni')
    G.add_edge('Bischeri', 'Guadagni')
    G.add_edge('Guadagni', 'Lamberteschi')
    return G


def les_miserables_graph():
    """Returns coappearance network of characters in the novel Les Miserables.

    References
    ----------
    .. [1] D. E. Knuth, 1993.
       The Stanford GraphBase: a platform for combinatorial computing,
       pp. 74-87. New York: AcM Press.
    """
    G = nx.Graph()
    G.add_edge('Napoleon', 'Myriel', weight=1)
    G.add_edge('MlleBaptistine', 'Myriel', weight=8)
    G.add_edge('MmeMagloire', 'Myriel', weight=10)
    G.add_edge('MmeMagloire', 'MlleBaptistine', weight=6)
    G.add_edge('CountessDeLo', 'Myriel', weight=1)
    G.add_edge('Geborand', 'Myriel', weight=1)
    G.add_edge('Champtercier', 'Myriel', weight=1)
    G.add_edge('Cravatte', 'Myriel', weight=1)
    G.add_edge('Count', 'Myriel', weight=2)
    G.add_edge('OldMan', 'Myriel', weight=1)
    G.add_edge('Valjean', 'Labarre', weight=1)
    G.add_edge('Valjean', 'MmeMagloire', weight=3)
    G.add_edge('Valjean', 'MlleBaptistine', weight=3)
    G.add_edge('Valjean', 'Myriel', weight=5)
    G.add_edge('Marguerite', 'Valjean', weight=1)
    G.add_edge('MmeDeR', 'Valjean', weight=1)
    G.add_edge('Isabeau', 'Valjean', weight=1)
    G.add_edge('Gervais', 'Valjean', weight=1)
    G.add_edge('Listolier', 'Tholomyes', weight=4)
    G.add_edge('Fameuil', 'Tholomyes', weight=4)
    G.add_edge('Fameuil', 'Listolier', weight=4)
    G.add_edge('Blacheville', 'Tholomyes', weight=4)
    G.add_edge('Blacheville', 'Listolier', weight=4)
    G.add_edge('Blacheville', 'Fameuil', weight=4)
    G.add_edge('Favourite', 'Tholomyes', weight=3)
    G.add_edge('Favourite', 'Listolier', weight=3)
    G.add_edge('Favourite', 'Fameuil', weight=3)
    G.add_edge('Favourite', 'Blacheville', weight=4)
    G.add_edge('Dahlia', 'Tholomyes', weight=3)
    G.add_edge('Dahlia', 'Listolier', weight=3)
    G.add_edge('Dahlia', 'Fameuil', weight=3)
    G.add_edge('Dahlia', 'Blacheville', weight=3)
    G.add_edge('Dahlia', 'Favourite', weight=5)
    G.add_edge('Zephine', 'Tholomyes', weight=3)
    G.add_edge('Zephine', 'Listolier', weight=3)
    G.add_edge('Zephine', 'Fameuil', weight=3)
    G.add_edge('Zephine', 'Blacheville', weight=3)
    G.add_edge('Zephine', 'Favourite', weight=4)
    G.add_edge('Zephine', 'Dahlia', weight=4)
    G.add_edge('Fantine', 'Tholomyes', weight=3)
    G.add_edge('Fantine', 'Listolier', weight=3)
    G.add_edge('Fantine', 'Fameuil', weight=3)
    G.add_edge('Fantine', 'Blacheville', weight=3)
    G.add_edge('Fantine', 'Favourite', weight=4)
    G.add_edge('Fantine', 'Dahlia', weight=4)
    G.add_edge('Fantine', 'Zephine', weight=4)
    G.add_edge('Fantine', 'Marguerite', weight=2)
    G.add_edge('Fantine', 'Valjean', weight=9)
    G.add_edge('MmeThenardier', 'Fantine', weight=2)
    G.add_edge('MmeThenardier', 'Valjean', weight=7)
    G.add_edge('Thenardier', 'MmeThenardier', weight=13)
    G.add_edge('Thenardier', 'Fantine', weight=1)
    G.add_edge('Thenardier', 'Valjean', weight=12)
    G.add_edge('Cosette', 'MmeThenardier', weight=4)
    G.add_edge('Cosette', 'Valjean', weight=31)
    G.add_edge('Cosette', 'Tholomyes', weight=1)
    G.add_edge('Cosette', 'Thenardier', weight=1)
    G.add_edge('Javert', 'Valjean', weight=17)
    G.add_edge('Javert', 'Fantine', weight=5)
    G.add_edge('Javert', 'Thenardier', weight=5)
    G.add_edge('Javert', 'MmeThenardier', weight=1)
    G.add_edge('Javert', 'Cosette', weight=1)
    G.add_edge('Fauchelevent', 'Valjean', weight=8)
    G.add_edge('Fauchelevent', 'Javert', weight=1)
    G.add_edge('Bamatabois', 'Fantine', weight=1)
    G.add_edge('Bamatabois', 'Javert', weight=1)
    G.add_edge('Bamatabois', 'Valjean', weight=2)
    G.add_edge('Perpetue', 'Fantine', weight=1)
    G.add_edge('Simplice', 'Perpetue', weight=2)
    G.add_edge('Simplice', 'Valjean', weight=3)
    G.add_edge('Simplice', 'Fantine', weight=2)
    G.add_edge('Simplice', 'Javert', weight=1)
    G.add_edge('Scaufflaire', 'Valjean', weight=1)
    G.add_edge('Woman1', 'Valjean', weight=2)
    G.add_edge('Woman1', 'Javert', weight=1)
    G.add_edge('Judge', 'Valjean', weight=3)
    G.add_edge('Judge', 'Bamatabois', weight=2)
    G.add_edge('Champmathieu', 'Valjean', weight=3)
    G.add_edge('Champmathieu', 'Judge', weight=3)
    G.add_edge('Champmathieu', 'Bamatabois', weight=2)
    G.add_edge('Brevet', 'Judge', weight=2)
    G.add_edge('Brevet', 'Champmathieu', weight=2)
    G.add_edge('Brevet', 'Valjean', weight=2)
    G.add_edge('Brevet', 'Bamatabois', weight=1)
    G.add_edge('Chenildieu', 'Judge', weight=2)
    G.add_edge('Chenildieu', 'Champmathieu', weight=2)
    G.add_edge('Chenildieu', 'Brevet', weight=2)
    G.add_edge('Chenildieu', 'Valjean', weight=2)
    G.add_edge('Chenildieu', 'Bamatabois', weight=1)
    G.add_edge('Cochepaille', 'Judge', weight=2)
    G.add_edge('Cochepaille', 'Champmathieu', weight=2)
    G.add_edge('Cochepaille', 'Brevet', weight=2)
    G.add_edge('Cochepaille', 'Chenildieu', weight=2)
    G.add_edge('Cochepaille', 'Valjean', weight=2)
    G.add_edge('Cochepaille', 'Bamatabois', weight=1)
    G.add_edge('Pontmercy', 'Thenardier', weight=1)
    G.add_edge('Boulatruelle', 'Thenardier', weight=1)
    G.add_edge('Eponine', 'MmeThenardier', weight=2)
    G.add_edge('Eponine', 'Thenardier', weight=3)
    G.add_edge('Anzelma', 'Eponine', weight=2)
    G.add_edge('Anzelma', 'Thenardier', weight=2)
    G.add_edge('Anzelma', 'MmeThenardier', weight=1)
    G.add_edge('Woman2', 'Valjean', weight=3)
    G.add_edge('Woman2', 'Cosette', weight=1)
    G.add_edge('Woman2', 'Javert', weight=1)
    G.add_edge('MotherInnocent', 'Fauchelevent', weight=3)
    G.add_edge('MotherInnocent', 'Valjean', weight=1)
    G.add_edge('Gribier', 'Fauchelevent', weight=2)
    G.add_edge('MmeBurgon', 'Jondrette', weight=1)
    G.add_edge('Gavroche', 'MmeBurgon', weight=2)
    G.add_edge('Gavroche', 'Thenardier', weight=1)
    G.add_edge('Gavroche', 'Javert', weight=1)
    G.add_edge('Gavroche', 'Valjean', weight=1)
    G.add_edge('Gillenormand', 'Cosette', weight=3)
    G.add_edge('Gillenormand', 'Valjean', weight=2)
    G.add_edge('Magnon', 'Gillenormand', weight=1)
    G.add_edge('Magnon', 'MmeThenardier', weight=1)
    G.add_edge('MlleGillenormand', 'Gillenormand', weight=9)
    G.add_edge('MlleGillenormand', 'Cosette', weight=2)
    G.add_edge('MlleGillenormand', 'Valjean', weight=2)
    G.add_edge('MmePontmercy', 'MlleGillenormand', weight=1)
    G.add_edge('MmePontmercy', 'Pontmercy', weight=1)
    G.add_edge('MlleVaubois', 'MlleGillenormand', weight=1)
    G.add_edge('LtGillenormand', 'MlleGillenormand', weight=2)
    G.add_edge('LtGillenormand', 'Gillenormand', weight=1)
    G.add_edge('LtGillenormand', 'Cosette', weight=1)
    G.add_edge('Marius', 'MlleGillenormand', weight=6)
    G.add_edge('Marius', 'Gillenormand', weight=12)
    G.add_edge('Marius', 'Pontmercy', weight=1)
    G.add_edge('Marius', 'LtGillenormand', weight=1)
    G.add_edge('Marius', 'Cosette', weight=21)
    G.add_edge('Marius', 'Valjean', weight=19)
    G.add_edge('Marius', 'Tholomyes', weight=1)
    G.add_edge('Marius', 'Thenardier', weight=2)
    G.add_edge('Marius', 'Eponine', weight=5)
    G.add_edge('Marius', 'Gavroche', weight=4)
    G.add_edge('BaronessT', 'Gillenormand', weight=1)
    G.add_edge('BaronessT', 'Marius', weight=1)
    G.add_edge('Mabeuf', 'Marius', weight=1)
    G.add_edge('Mabeuf', 'Eponine', weight=1)
    G.add_edge('Mabeuf', 'Gavroche', weight=1)
    G.add_edge('Enjolras', 'Marius', weight=7)
    G.add_edge('Enjolras', 'Gavroche', weight=7)
    G.add_edge('Enjolras', 'Javert', weight=6)
    G.add_edge('Enjolras', 'Mabeuf', weight=1)
    G.add_edge('Enjolras', 'Valjean', weight=4)
    G.add_edge('Combeferre', 'Enjolras', weight=15)
    G.add_edge('Combeferre', 'Marius', weight=5)
    G.add_edge('Combeferre', 'Gavroche', weight=6)
    G.add_edge('Combeferre', 'Mabeuf', weight=2)
    G.add_edge('Prouvaire', 'Gavroche', weight=1)
    G.add_edge('Prouvaire', 'Enjolras', weight=4)
    G.add_edge('Prouvaire', 'Combeferre', weight=2)
    G.add_edge('Feuilly', 'Gavroche', weight=2)
    G.add_edge('Feuilly', 'Enjolras', weight=6)
    G.add_edge('Feuilly', 'Prouvaire', weight=2)
    G.add_edge('Feuilly', 'Combeferre', weight=5)
    G.add_edge('Feuilly', 'Mabeuf', weight=1)
    G.add_edge('Feuilly', 'Marius', weight=1)
    G.add_edge('Courfeyrac', 'Marius', weight=9)
    G.add_edge('Courfeyrac', 'Enjolras', weight=17)
    G.add_edge('Courfeyrac', 'Combeferre', weight=13)
    G.add_edge('Courfeyrac', 'Gavroche', weight=7)
    G.add_edge('Courfeyrac', 'Mabeuf', weight=2)
    G.add_edge('Courfeyrac', 'Eponine', weight=1)
    G.add_edge('Courfeyrac', 'Feuilly', weight=6)
    G.add_edge('Courfeyrac', 'Prouvaire', weight=3)
    G.add_edge('Bahorel', 'Combeferre', weight=5)
    G.add_edge('Bahorel', 'Gavroche', weight=5)
    G.add_edge('Bahorel', 'Courfeyrac', weight=6)
    G.add_edge('Bahorel', 'Mabeuf', weight=2)
    G.add_edge('Bahorel', 'Enjolras', weight=4)
    G.add_edge('Bahorel', 'Feuilly', weight=3)
    G.add_edge('Bahorel', 'Prouvaire', weight=2)
    G.add_edge('Bahorel', 'Marius', weight=1)
    G.add_edge('Bossuet', 'Marius', weight=5)
    G.add_edge('Bossuet', 'Courfeyrac', weight=12)
    G.add_edge('Bossuet', 'Gavroche', weight=5)
    G.add_edge('Bossuet', 'Bahorel', weight=4)
    G.add_edge('Bossuet', 'Enjolras', weight=10)
    G.add_edge('Bossuet', 'Feuilly', weight=6)
    G.add_edge('Bossuet', 'Prouvaire', weight=2)
    G.add_edge('Bossuet', 'Combeferre', weight=9)
    G.add_edge('Bossuet', 'Mabeuf', weight=1)
    G.add_edge('Bossuet', 'Valjean', weight=1)
    G.add_edge('Joly', 'Bahorel', weight=5)
    G.add_edge('Joly', 'Bossuet', weight=7)
    G.add_edge('Joly', 'Gavroche', weight=3)
    G.add_edge('Joly', 'Courfeyrac', weight=5)
    G.add_edge('Joly', 'Enjolras', weight=5)
    G.add_edge('Joly', 'Feuilly', weight=5)
    G.add_edge('Joly', 'Prouvaire', weight=2)
    G.add_edge('Joly', 'Combeferre', weight=5)
    G.add_edge('Joly', 'Mabeuf', weight=1)
    G.add_edge('Joly', 'Marius', weight=2)
    G.add_edge('Grantaire', 'Bossuet', weight=3)
    G.add_edge('Grantaire', 'Enjolras', weight=3)
    G.add_edge('Grantaire', 'Combeferre', weight=1)
    G.add_edge('Grantaire', 'Courfeyrac', weight=2)
    G.add_edge('Grantaire', 'Joly', weight=2)
    G.add_edge('Grantaire', 'Gavroche', weight=1)
    G.add_edge('Grantaire', 'Bahorel', weight=1)
    G.add_edge('Grantaire', 'Feuilly', weight=1)
    G.add_edge('Grantaire', 'Prouvaire', weight=1)
    G.add_edge('MotherPlutarch', 'Mabeuf', weight=3)
    G.add_edge('Gueulemer', 'Thenardier', weight=5)
    G.add_edge('Gueulemer', 'Valjean', weight=1)
    G.add_edge('Gueulemer', 'MmeThenardier', weight=1)
    G.add_edge('Gueulemer', 'Javert', weight=1)
    G.add_edge('Gueulemer', 'Gavroche', weight=1)
    G.add_edge('Gueulemer', 'Eponine', weight=1)
    G.add_edge('Babet', 'Thenardier', weight=6)
    G.add_edge('Babet', 'Gueulemer', weight=6)
    G.add_edge('Babet', 'Valjean', weight=1)
    G.add_edge('Babet', 'MmeThenardier', weight=1)
    G.add_edge('Babet', 'Javert', weight=2)
    G.add_edge('Babet', 'Gavroche', weight=1)
    G.add_edge('Babet', 'Eponine', weight=1)
    G.add_edge('Claquesous', 'Thenardier', weight=4)
    G.add_edge('Claquesous', 'Babet', weight=4)
    G.add_edge('Claquesous', 'Gueulemer', weight=4)
    G.add_edge('Claquesous', 'Valjean', weight=1)
    G.add_edge('Claquesous', 'MmeThenardier', weight=1)
    G.add_edge('Claquesous', 'Javert', weight=1)
    G.add_edge('Claquesous', 'Eponine', weight=1)
    G.add_edge('Claquesous', 'Enjolras', weight=1)
    G.add_edge('Montparnasse', 'Javert', weight=1)
    G.add_edge('Montparnasse', 'Babet', weight=2)
    G.add_edge('Montparnasse', 'Gueulemer', weight=2)
    G.add_edge('Montparnasse', 'Claquesous', weight=2)
    G.add_edge('Montparnasse', 'Valjean', weight=1)
    G.add_edge('Montparnasse', 'Gavroche', weight=1)
    G.add_edge('Montparnasse', 'Eponine', weight=1)
    G.add_edge('Montparnasse', 'Thenardier', weight=1)
    G.add_edge('Toussaint', 'Cosette', weight=2)
    G.add_edge('Toussaint', 'Javert', weight=1)
    G.add_edge('Toussaint', 'Valjean', weight=1)
    G.add_edge('Child1', 'Gavroche', weight=2)
    G.add_edge('Child2', 'Gavroche', weight=2)
    G.add_edge('Child2', 'Child1', weight=3)
    G.add_edge('Brujon', 'Babet', weight=3)
    G.add_edge('Brujon', 'Gueulemer', weight=3)
    G.add_edge('Brujon', 'Thenardier', weight=3)
    G.add_edge('Brujon', 'Gavroche', weight=1)
    G.add_edge('Brujon', 'Eponine', weight=1)
    G.add_edge('Brujon', 'Claquesous', weight=1)
    G.add_edge('Brujon', 'Montparnasse', weight=1)
    G.add_edge('MmeHucheloup', 'Bossuet', weight=1)
    G.add_edge('MmeHucheloup', 'Joly', weight=1)
    G.add_edge('MmeHucheloup', 'Grantaire', weight=1)
    G.add_edge('MmeHucheloup', 'Bahorel', weight=1)
    G.add_edge('MmeHucheloup', 'Courfeyrac', weight=1)
    G.add_edge('MmeHucheloup', 'Gavroche', weight=1)
    G.add_edge('MmeHucheloup', 'Enjolras', weight=1)
    return G



#!/usr/bin/env python
"""
================
Degree histogram
================

Draw degree histogram with matplotlib.
Random graph shown as inset
"""
import collections
import matplotlib.pyplot as plt
import networkx as nx

G = nx.gnp_random_graph(100, 0.02)

degree_sequence = sorted([d for n, d in G.degree()], reverse=True)  # degree sequence
# print "Degree sequence", degree_sequence
degreeCount = collections.Counter(degree_sequence)
deg, cnt = zip(*degreeCount.items())

fig, ax = plt.subplots()
plt.bar(deg, cnt, width=0.80, color='b')

plt.title("Degree Histogram")
plt.ylabel("Count")
plt.xlabel("Degree")
ax.set_xticks([d + 0.4 for d in deg])
ax.set_xticklabels(deg)

# draw graph in inset
plt.axes([0.4, 0.4, 0.5, 0.5])
Gcc = G.subgraph(sorted(nx.connected_components(G), key=len, reverse=True)[0])
pos = nx.spring_layout(G)
plt.axis('off')
nx.draw_networkx_nodes(G, pos, node_size=20)
nx.draw_networkx_edges(G, pos, alpha=0.4)

plt.show()



#!/usr/bin/env python
import pytest
import networkx as nx

#import random
#random.seed(0)


def test_double_edge_swap():
    graph = nx.barabasi_albert_graph(200, 1)
    degrees = sorted(d for n, d in graph.degree())
    G = nx.double_edge_swap(graph, 40)
    assert degrees == sorted(d for n, d in graph.degree())


def test_double_edge_swap_seed():
    graph = nx.barabasi_albert_graph(200, 1)
    degrees = sorted(d for n, d in graph.degree())
    G = nx.double_edge_swap(graph, 40, seed=1)
    assert degrees == sorted(d for n, d in graph.degree())


def test_connected_double_edge_swap():
    graph = nx.barabasi_albert_graph(200, 1)
    degrees = sorted(d for n, d in graph.degree())
    G = nx.connected_double_edge_swap(graph, 40, seed=1)
    assert nx.is_connected(graph)
    assert degrees == sorted(d for n, d in graph.degree())


def test_double_edge_swap_small():
    with pytest.raises(nx.NetworkXError):
        G = nx.double_edge_swap(nx.path_graph(3))


def test_double_edge_swap_tries():
    with pytest.raises(nx.NetworkXError):
        G = nx.double_edge_swap(nx.path_graph(10), nswap=1, max_tries=0)


def test_connected_double_edge_swap_small():
    with pytest.raises(nx.NetworkXError):
        G = nx.connected_double_edge_swap(nx.path_graph(3))


def test_connected_double_edge_swap_not_connected():
    with pytest.raises(nx.NetworkXError):
        G = nx.path_graph(3)
        nx.add_path(G, [10, 11, 12])
        G = nx.connected_double_edge_swap(G)


def test_degree_seq_c4():
    G = nx.cycle_graph(4)
    degrees = sorted(d for n, d in G.degree())
    G = nx.double_edge_swap(G, 1, 100)
    assert degrees == sorted(d for n, d in G.degree())



#!/usr/bin/env python
"""
Threshold Graphs
================
"""

import pytest

import networkx as nx
import networkx.algorithms.threshold as nxt
from networkx.algorithms.isomorphism.isomorph import graph_could_be_isomorphic
from networkx.testing import almost_equal

cnlti = nx.convert_node_labels_to_integers


class TestGeneratorThreshold():
    def test_threshold_sequence_graph_test(self):
        G = nx.star_graph(10)
        assert nxt.is_threshold_graph(G)
        assert nxt.is_threshold_sequence(list(d for n, d in G.degree()))

        G = nx.complete_graph(10)
        assert nxt.is_threshold_graph(G)
        assert nxt.is_threshold_sequence(list(d for n, d in G.degree()))

        deg = [3, 2, 2, 1, 1, 1]
        assert not nxt.is_threshold_sequence(deg)

        deg = [3, 2, 2, 1]
        assert nxt.is_threshold_sequence(deg)

        G = nx.generators.havel_hakimi_graph(deg)
        assert nxt.is_threshold_graph(G)

    def test_creation_sequences(self):
        deg = [3, 2, 2, 1]
        G = nx.generators.havel_hakimi_graph(deg)

        with pytest.raises(ValueError):
            nxt.creation_sequence(deg, with_labels=True, compact=True)

        cs0 = nxt.creation_sequence(deg)
        H0 = nxt.threshold_graph(cs0)
        assert ''.join(cs0) == 'ddid'

        cs1 = nxt.creation_sequence(deg, with_labels=True)
        H1 = nxt.threshold_graph(cs1)
        assert cs1 == [(1, 'd'), (2, 'd'), (3, 'i'), (0, 'd')]

        cs2 = nxt.creation_sequence(deg, compact=True)
        H2 = nxt.threshold_graph(cs2)
        assert cs2 == [2, 1, 1]
        assert ''.join(nxt.uncompact(cs2)) == 'ddid'
        assert graph_could_be_isomorphic(H0, G)
        assert graph_could_be_isomorphic(H0, H1)
        assert graph_could_be_isomorphic(H0, H2)

    def test_make_compact(self):
        assert nxt.make_compact(['d', 'd', 'd', 'i', 'd', 'd']) == [3, 1, 2]
        assert nxt.make_compact([3, 1, 2]) == [3, 1, 2]
        assert pytest.raises(TypeError, nxt.make_compact, [3., 1., 2.])

    def test_uncompact(self):
        assert nxt.uncompact([3, 1, 2]) == ['d', 'd', 'd', 'i', 'd', 'd']
        assert nxt.uncompact(['d', 'd', 'i', 'd']) == ['d', 'd', 'i', 'd']
        assert (nxt.uncompact(nxt.uncompact([(1, 'd'), (2, 'd'), (3, 'i'), (0, 'd')])) ==
                     nxt.uncompact([(1, 'd'), (2, 'd'), (3, 'i'), (0, 'd')]))
        assert pytest.raises(TypeError, nxt.uncompact, [3., 1., 2.])

    def test_creation_sequence_to_weights(self):
        assert nxt.creation_sequence_to_weights([3, 1, 2]) == [0.5, 0.5, 0.5, 0.25, 0.75, 0.75]
        assert pytest.raises(TypeError, nxt.creation_sequence_to_weights, [3., 1., 2.])

    def test_weights_to_creation_sequence(self):
        deg = [3, 2, 2, 1]
        with pytest.raises(ValueError):
            nxt.weights_to_creation_sequence(deg, with_labels=True, compact=True)
        assert (nxt.weights_to_creation_sequence(deg, with_labels=True) ==
                     [(3, 'd'), (1, 'd'), (2, 'd'), (0, 'd')])
        assert nxt.weights_to_creation_sequence(deg, compact=True) == [4]

    def test_find_alternating_4_cycle(self):
        G = nx.Graph()
        G.add_edge(1, 2)
        assert not nxt.find_alternating_4_cycle(G)

    def test_shortest_path(self):
        deg = [3, 2, 2, 1]
        G = nx.generators.havel_hakimi_graph(deg)
        cs1 = nxt.creation_sequence(deg, with_labels=True)
        for n, m in [(3, 0), (0, 3), (0, 2), (0, 1), (1, 3),
                     (3, 1), (1, 2), (2, 3)]:
            assert (nxt.shortest_path(cs1, n, m) ==
                         nx.shortest_path(G, n, m))

        spl = nxt.shortest_path_length(cs1, 3)
        spl2 = nxt.shortest_path_length([t for v, t in cs1], 2)
        assert spl == spl2

        spld = {}
        for j, pl in enumerate(spl):
            n = cs1[j][0]
            spld[n] = pl
        assert spld == nx.single_source_shortest_path_length(G, 3)

        assert nxt.shortest_path(['d', 'd', 'd', 'i', 'd', 'd'], 1, 2) == [1, 2]
        assert nxt.shortest_path([3, 1, 2], 1, 2) == [1, 2]
        assert pytest.raises(TypeError, nxt.shortest_path, [3., 1., 2.], 1, 2)
        assert pytest.raises(ValueError, nxt.shortest_path, [3, 1, 2], 'a', 2)
        assert pytest.raises(ValueError, nxt.shortest_path, [3, 1, 2], 1, 'b')
        assert nxt.shortest_path([3, 1, 2], 1, 1) == [1]

    def test_shortest_path_length(self):
        assert nxt.shortest_path_length([3, 1, 2], 1) == [1, 0, 1, 2, 1, 1]
        assert (nxt.shortest_path_length(['d', 'd', 'd', 'i', 'd', 'd'], 1) ==
                     [1, 0, 1, 2, 1, 1])
        assert (nxt.shortest_path_length(('d', 'd', 'd', 'i', 'd', 'd'), 1) ==
                     [1, 0, 1, 2, 1, 1])
        assert pytest.raises(TypeError, nxt.shortest_path, [3., 1., 2.], 1)

    def random_threshold_sequence(self):
        assert len(nxt.random_threshold_sequence(10, 0.5)) == 10
        assert (nxt.random_threshold_sequence(10, 0.5, seed=42) ==
                     ['d', 'i', 'd', 'd', 'd', 'i', 'i', 'i', 'd', 'd'])
        assert pytest.raises(ValueError, nxt.random_threshold_sequence, 10, 1.5)

    def test_right_d_threshold_sequence(self):
        assert nxt.right_d_threshold_sequence(3, 2) == ['d', 'i', 'd']
        assert pytest.raises(ValueError, nxt.right_d_threshold_sequence, 2, 3)

    def test_left_d_threshold_sequence(self):
        assert nxt.left_d_threshold_sequence(3, 2) == ['d', 'i', 'd']
        assert pytest.raises(ValueError, nxt.left_d_threshold_sequence, 2, 3)

    def test_weights_thresholds(self):
        wseq = [3, 4, 3, 3, 5, 6, 5, 4, 5, 6]
        cs = nxt.weights_to_creation_sequence(wseq, threshold=10)
        wseq = nxt.creation_sequence_to_weights(cs)
        cs2 = nxt.weights_to_creation_sequence(wseq)
        assert cs == cs2

        wseq = nxt.creation_sequence_to_weights(nxt.uncompact([3, 1, 2, 3, 3, 2, 3]))
        assert (wseq ==
                     [s * 0.125 for s in [4, 4, 4, 3, 5, 5, 2, 2, 2, 6, 6, 6, 1, 1, 7, 7, 7]])

        wseq = nxt.creation_sequence_to_weights([3, 1, 2, 3, 3, 2, 3])
        assert (wseq ==
                     [s * 0.125 for s in [4, 4, 4, 3, 5, 5, 2, 2, 2, 6, 6, 6, 1, 1, 7, 7, 7]])

        wseq = nxt.creation_sequence_to_weights(list(enumerate('ddidiiidididi')))
        assert (wseq ==
                     [s * 0.1 for s in [5, 5, 4, 6, 3, 3, 3, 7, 2, 8, 1, 9, 0]])

        wseq = nxt.creation_sequence_to_weights('ddidiiidididi')
        assert (wseq ==
                     [s * 0.1 for s in [5, 5, 4, 6, 3, 3, 3, 7, 2, 8, 1, 9, 0]])

        wseq = nxt.creation_sequence_to_weights('ddidiiidididid')
        ws = [s / float(12) for s in [6, 6, 5, 7, 4, 4, 4, 8, 3, 9, 2, 10, 1, 11]]
        assert sum([abs(c - d) for c, d in zip(wseq, ws)]) < 1e-14

    def test_finding_routines(self):
        G = nx.Graph({1: [2], 2: [3], 3: [4], 4: [5], 5: [6]})
        G.add_edge(2, 4)
        G.add_edge(2, 5)
        G.add_edge(2, 7)
        G.add_edge(3, 6)
        G.add_edge(4, 6)

        # Alternating 4 cycle
        assert nxt.find_alternating_4_cycle(G) == [1, 2, 3, 6]

        # Threshold graph
        TG = nxt.find_threshold_graph(G)
        assert nxt.is_threshold_graph(TG)
        assert sorted(TG.nodes()) == [1, 2, 3, 4, 5, 7]

        cs = nxt.creation_sequence(dict(TG.degree()), with_labels=True)
        assert nxt.find_creation_sequence(G) == cs

    def test_fast_versions_properties_threshold_graphs(self):
        cs = 'ddiiddid'
        G = nxt.threshold_graph(cs)
        assert nxt.density('ddiiddid') == nx.density(G)
        assert (sorted(nxt.degree_sequence(cs)) ==
                     sorted(d for n, d in G.degree()))

        ts = nxt.triangle_sequence(cs)
        assert ts == list(nx.triangles(G).values())
        assert sum(ts) // 3 == nxt.triangles(cs)

        c1 = nxt.cluster_sequence(cs)
        c2 = list(nx.clustering(G).values())
        assert almost_equal(sum([abs(c - d) for c, d in zip(c1, c2)]), 0)

        b1 = nx.betweenness_centrality(G).values()
        b2 = nxt.betweenness_sequence(cs)
        assert sum([abs(c - d) for c, d in zip(b1, b2)]) < 1e-14

        assert nxt.eigenvalues(cs) == [0, 1, 3, 3, 5, 7, 7, 8]

        # Degree Correlation
        assert abs(nxt.degree_correlation(cs) + 0.593038821954) < 1e-12
        assert nxt.degree_correlation('diiiddi') == -0.8
        assert nxt.degree_correlation('did') == -1.0
        assert nxt.degree_correlation('ddd') == 1.0
        assert nxt.eigenvalues('dddiii') == [0, 0, 0, 0, 3, 3]
        assert nxt.eigenvalues('dddiiid') == [0, 1, 1, 1, 4, 4, 7]

    def test_tg_creation_routines(self):
        s = nxt.left_d_threshold_sequence(5, 7)
        s = nxt.right_d_threshold_sequence(5, 7)
        s1 = nxt.swap_d(s, 1.0, 1.0)
        s1 = nxt.swap_d(s, 1.0, 1.0, seed=1)

    def test_eigenvectors(self):
        np = pytest.importorskip('numpy')
        eigenval = np.linalg.eigvals
        scipy = pytest.importorskip('scipy')

        cs = 'ddiiddid'
        G = nxt.threshold_graph(cs)
        (tgeval, tgevec) = nxt.eigenvectors(cs)
        dot = np.dot
        assert [abs(dot(lv, lv) - 1.0) < 1e-9 for lv in tgevec] == [True] * 8
        lapl = nx.laplacian_matrix(G)
#        tgev=[ dot(lv,dot(lapl,lv)) for lv in tgevec ]
#        assert_true(sum([abs(c-d) for c,d in zip(tgev,tgeval)]) < 1e-9)
#        tgev.sort()
#        lev=list(eigenval(lapl))
#        lev.sort()
#        assert_true(sum([abs(c-d) for c,d in zip(tgev,lev)]) < 1e-9)

    def test_create_using(self):
        cs = 'ddiiddid'
        G = nxt.threshold_graph(cs)
        assert pytest.raises(nx.exception.NetworkXError,
                      nxt.threshold_graph, cs, create_using=nx.DiGraph())
        MG = nxt.threshold_graph(cs, create_using=nx.MultiGraph())
        assert sorted(MG.edges()) == sorted(G.edges())



#!/usr/bin/env python
"""
=====================
Krackhardt Centrality
=====================

Centrality measures of Krackhardt social network.
"""
# Author: Aric Hagberg (hagberg@lanl.gov)
# Date: 2005-05-12 14:33:11 -0600 (Thu, 12 May 2005)
# Revision: 998

#    Copyright (C) 2004-2019 by
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    All rights reserved.
#    BSD license.

import matplotlib.pyplot as plt
import networkx as nx

G = nx.krackhardt_kite_graph()

print("Betweenness")
b = nx.betweenness_centrality(G)
for v in G.nodes():
    print("%0.2d %5.3f" % (v, b[v]))

print("Degree centrality")
d = nx.degree_centrality(G)
for v in G.nodes():
    print("%0.2d %5.3f" % (v, d[v]))

print("Closeness centrality")
c = nx.closeness_centrality(G)
for v in G.nodes():
    print("%0.2d %5.3f" % (v, c[v]))

nx.draw(G)
plt.show()



#!/usr/bin/env python

import pytest
import networkx as nx
from networkx.algorithms.bipartite.generators import *

"""Generators - Bipartite
----------------------
"""


class TestGeneratorsBipartite():
    def test_complete_bipartite_graph(self):
        G = complete_bipartite_graph(0, 0)
        assert nx.is_isomorphic(G, nx.null_graph())

        for i in [1, 5]:
            G = complete_bipartite_graph(i, 0)
            assert nx.is_isomorphic(G, nx.empty_graph(i))
            G = complete_bipartite_graph(0, i)
            assert nx.is_isomorphic(G, nx.empty_graph(i))

        G = complete_bipartite_graph(2, 2)
        assert nx.is_isomorphic(G, nx.cycle_graph(4))

        G = complete_bipartite_graph(1, 5)
        assert nx.is_isomorphic(G, nx.star_graph(5))

        G = complete_bipartite_graph(5, 1)
        assert nx.is_isomorphic(G, nx.star_graph(5))

        # complete_bipartite_graph(m1,m2) is a connected graph with
        # m1+m2 nodes and  m1*m2 edges
        for m1, m2 in [(5, 11), (7, 3)]:
            G = complete_bipartite_graph(m1, m2)
            assert nx.number_of_nodes(G) == m1 + m2
            assert nx.number_of_edges(G) == m1 * m2

        pytest.raises(nx.NetworkXError, complete_bipartite_graph,
                      7, 3, create_using=nx.DiGraph)
        pytest.raises(nx.NetworkXError, complete_bipartite_graph,
                      7, 3, create_using=nx.DiGraph)
        pytest.raises(nx.NetworkXError, complete_bipartite_graph,
                      7, 3, create_using=nx.MultiDiGraph)

        mG = complete_bipartite_graph(7, 3, create_using=nx.MultiGraph)
        assert mG.is_multigraph()
        assert sorted(mG.edges()) == sorted(G.edges())

        mG = complete_bipartite_graph(7, 3, create_using=nx.MultiGraph)
        assert mG.is_multigraph()
        assert sorted(mG.edges()) == sorted(G.edges())

        mG = complete_bipartite_graph(7, 3)  # default to Graph
        assert sorted(mG.edges()) == sorted(G.edges())
        assert not mG.is_multigraph()
        assert not mG.is_directed()

        # specify nodes rather than number of nodes
        G = complete_bipartite_graph([1, 2], ['a', 'b'])
        has_edges = G.has_edge(1, 'a') & G.has_edge(1, 'b') &\
            G.has_edge(2, 'a') & G.has_edge(2, 'b')
        assert has_edges
        assert G.size() == 4

    def test_configuration_model(self):
        aseq = []
        bseq = []
        G = configuration_model(aseq, bseq)
        assert len(G) == 0

        aseq = [0, 0]
        bseq = [0, 0]
        G = configuration_model(aseq, bseq)
        assert len(G) == 4
        assert G.number_of_edges() == 0

        aseq = [3, 3, 3, 3]
        bseq = [2, 2, 2, 2, 2]
        pytest.raises(nx.NetworkXError,
                      configuration_model, aseq, bseq)

        aseq = [3, 3, 3, 3]
        bseq = [2, 2, 2, 2, 2, 2]
        G = configuration_model(aseq, bseq)
        assert (sorted(d for n, d in G.degree()) ==
                     [2, 2, 2, 2, 2, 2, 3, 3, 3, 3])

        aseq = [2, 2, 2, 2, 2, 2]
        bseq = [3, 3, 3, 3]
        G = configuration_model(aseq, bseq)
        assert (sorted(d for n, d in G.degree()) ==
                     [2, 2, 2, 2, 2, 2, 3, 3, 3, 3])

        aseq = [2, 2, 2, 1, 1, 1]
        bseq = [3, 3, 3]
        G = configuration_model(aseq, bseq)
        assert G.is_multigraph()
        assert not G.is_directed()
        assert (sorted(d for n, d in G.degree()) ==
                     [1, 1, 1, 2, 2, 2, 3, 3, 3])

        GU = nx.project(nx.Graph(G), range(len(aseq)))
        assert GU.number_of_nodes() == 6

        GD = nx.project(nx.Graph(G), range(len(aseq), len(aseq) + len(bseq)))
        assert GD.number_of_nodes() == 3

        G = reverse_havel_hakimi_graph(aseq, bseq, create_using=nx.Graph)
        assert not G.is_multigraph()
        assert not G.is_directed()

        pytest.raises(nx.NetworkXError,
                      configuration_model, aseq, bseq,
                      create_using=nx.DiGraph())
        pytest.raises(nx.NetworkXError,
                      configuration_model, aseq, bseq,
                      create_using=nx.DiGraph)
        pytest.raises(nx.NetworkXError,
                      configuration_model, aseq, bseq,
                      create_using=nx.MultiDiGraph)

    def test_havel_hakimi_graph(self):
        aseq = []
        bseq = []
        G = havel_hakimi_graph(aseq, bseq)
        assert len(G) == 0

        aseq = [0, 0]
        bseq = [0, 0]
        G = havel_hakimi_graph(aseq, bseq)
        assert len(G) == 4
        assert G.number_of_edges() == 0

        aseq = [3, 3, 3, 3]
        bseq = [2, 2, 2, 2, 2]
        pytest.raises(nx.NetworkXError,
                      havel_hakimi_graph, aseq, bseq)

        bseq = [2, 2, 2, 2, 2, 2]
        G = havel_hakimi_graph(aseq, bseq)
        assert (sorted(d for n, d in G.degree()) ==
                     [2, 2, 2, 2, 2, 2, 3, 3, 3, 3])

        aseq = [2, 2, 2, 2, 2, 2]
        bseq = [3, 3, 3, 3]
        G = havel_hakimi_graph(aseq, bseq)
        assert G.is_multigraph()
        assert not G.is_directed()
        assert (sorted(d for n, d in G.degree()) ==
                     [2, 2, 2, 2, 2, 2, 3, 3, 3, 3])

        GU = nx.project(nx.Graph(G), range(len(aseq)))
        assert GU.number_of_nodes() == 6

        GD = nx.project(nx.Graph(G), range(len(aseq), len(aseq) + len(bseq)))
        assert GD.number_of_nodes() == 4

        G = reverse_havel_hakimi_graph(aseq, bseq, create_using=nx.Graph)
        assert not G.is_multigraph()
        assert not G.is_directed()

        pytest.raises(nx.NetworkXError,
                      havel_hakimi_graph, aseq, bseq,
                      create_using=nx.DiGraph)
        pytest.raises(nx.NetworkXError,
                      havel_hakimi_graph, aseq, bseq,
                      create_using=nx.DiGraph)
        pytest.raises(nx.NetworkXError,
                      havel_hakimi_graph, aseq, bseq,
                      create_using=nx.MultiDiGraph)

    def test_reverse_havel_hakimi_graph(self):
        aseq = []
        bseq = []
        G = reverse_havel_hakimi_graph(aseq, bseq)
        assert len(G) == 0

        aseq = [0, 0]
        bseq = [0, 0]
        G = reverse_havel_hakimi_graph(aseq, bseq)
        assert len(G) == 4
        assert G.number_of_edges() == 0

        aseq = [3, 3, 3, 3]
        bseq = [2, 2, 2, 2, 2]
        pytest.raises(nx.NetworkXError,
                      reverse_havel_hakimi_graph, aseq, bseq)

        bseq = [2, 2, 2, 2, 2, 2]
        G = reverse_havel_hakimi_graph(aseq, bseq)
        assert (sorted(d for n, d in G.degree()) ==
                     [2, 2, 2, 2, 2, 2, 3, 3, 3, 3])

        aseq = [2, 2, 2, 2, 2, 2]
        bseq = [3, 3, 3, 3]
        G = reverse_havel_hakimi_graph(aseq, bseq)
        assert (sorted(d for n, d in G.degree()) ==
                     [2, 2, 2, 2, 2, 2, 3, 3, 3, 3])

        aseq = [2, 2, 2, 1, 1, 1]
        bseq = [3, 3, 3]
        G = reverse_havel_hakimi_graph(aseq, bseq)
        assert G.is_multigraph()
        assert not G.is_directed()
        assert (sorted(d for n, d in G.degree()) ==
                     [1, 1, 1, 2, 2, 2, 3, 3, 3])

        GU = nx.project(nx.Graph(G), range(len(aseq)))
        assert GU.number_of_nodes() == 6

        GD = nx.project(nx.Graph(G), range(len(aseq), len(aseq) + len(bseq)))
        assert GD.number_of_nodes() == 3

        G = reverse_havel_hakimi_graph(aseq, bseq, create_using=nx.Graph)
        assert not G.is_multigraph()
        assert not G.is_directed()

        pytest.raises(nx.NetworkXError,
                      reverse_havel_hakimi_graph, aseq, bseq,
                      create_using=nx.DiGraph)
        pytest.raises(nx.NetworkXError,
                      reverse_havel_hakimi_graph, aseq, bseq,
                      create_using=nx.DiGraph)
        pytest.raises(nx.NetworkXError,
                      reverse_havel_hakimi_graph, aseq, bseq,
                      create_using=nx.MultiDiGraph)

    def test_alternating_havel_hakimi_graph(self):
        aseq = []
        bseq = []
        G = alternating_havel_hakimi_graph(aseq, bseq)
        assert len(G) == 0

        aseq = [0, 0]
        bseq = [0, 0]
        G = alternating_havel_hakimi_graph(aseq, bseq)
        assert len(G) == 4
        assert G.number_of_edges() == 0

        aseq = [3, 3, 3, 3]
        bseq = [2, 2, 2, 2, 2]
        pytest.raises(nx.NetworkXError,
                      alternating_havel_hakimi_graph, aseq, bseq)

        bseq = [2, 2, 2, 2, 2, 2]
        G = alternating_havel_hakimi_graph(aseq, bseq)
        assert (sorted(d for n, d in G.degree()) ==
                     [2, 2, 2, 2, 2, 2, 3, 3, 3, 3])

        aseq = [2, 2, 2, 2, 2, 2]
        bseq = [3, 3, 3, 3]
        G = alternating_havel_hakimi_graph(aseq, bseq)
        assert (sorted(d for n, d in G.degree()) ==
                     [2, 2, 2, 2, 2, 2, 3, 3, 3, 3])

        aseq = [2, 2, 2, 1, 1, 1]
        bseq = [3, 3, 3]
        G = alternating_havel_hakimi_graph(aseq, bseq)
        assert G.is_multigraph()
        assert not G.is_directed()
        assert (sorted(d for n, d in G.degree()) ==
                     [1, 1, 1, 2, 2, 2, 3, 3, 3])

        GU = nx.project(nx.Graph(G), range(len(aseq)))
        assert GU.number_of_nodes() == 6

        GD = nx.project(nx.Graph(G), range(len(aseq), len(aseq) + len(bseq)))
        assert GD.number_of_nodes() == 3

        G = reverse_havel_hakimi_graph(aseq, bseq, create_using=nx.Graph)
        assert not G.is_multigraph()
        assert not G.is_directed()

        pytest.raises(nx.NetworkXError,
                      alternating_havel_hakimi_graph, aseq, bseq,
                      create_using=nx.DiGraph)
        pytest.raises(nx.NetworkXError,
                      alternating_havel_hakimi_graph, aseq, bseq,
                      create_using=nx.DiGraph)
        pytest.raises(nx.NetworkXError,
                      alternating_havel_hakimi_graph, aseq, bseq,
                      create_using=nx.MultiDiGraph)

    def test_preferential_attachment(self):
        aseq = [3, 2, 1, 1]
        G = preferential_attachment_graph(aseq, 0.5)
        assert G.is_multigraph()
        assert not G.is_directed()

        G = preferential_attachment_graph(aseq, 0.5, create_using=nx.Graph)
        assert not G.is_multigraph()
        assert not G.is_directed()

        pytest.raises(nx.NetworkXError,
                      preferential_attachment_graph, aseq, 0.5,
                      create_using=nx.DiGraph())
        pytest.raises(nx.NetworkXError,
                      preferential_attachment_graph, aseq, 0.5,
                      create_using=nx.DiGraph())
        pytest.raises(nx.NetworkXError,
                      preferential_attachment_graph, aseq, 0.5,
                      create_using=nx.DiGraph())

    def test_random_graph(self):
        n = 10
        m = 20
        G = random_graph(n, m, 0.9)
        assert len(G) == 30
        assert nx.is_bipartite(G)
        X, Y = nx.algorithms.bipartite.sets(G)
        assert set(range(n)) == X
        assert set(range(n, n + m)) == Y

    def test_random_digraph(self):
        n = 10
        m = 20
        G = random_graph(n, m, 0.9, directed=True)
        assert len(G) == 30
        assert nx.is_bipartite(G)
        X, Y = nx.algorithms.bipartite.sets(G)
        assert set(range(n)) == X
        assert set(range(n, n + m)) == Y

    def test_gnmk_random_graph(self):
        n = 10
        m = 20
        edges = 100
        # set seed because sometimes it is not connected
        # which raises an error in bipartite.sets(G) below.
        G = gnmk_random_graph(n, m, edges, seed=1234)
        assert len(G) == n + m
        assert nx.is_bipartite(G)
        X, Y = nx.algorithms.bipartite.sets(G)
        #print(X)
        assert set(range(n)) == X
        assert set(range(n, n + m)) == Y
        assert edges == len(list(G.edges()))

    def test_gnmk_random_graph_complete(self):
        n = 10
        m = 20
        edges = 200
        G = gnmk_random_graph(n, m, edges)
        assert len(G) == n + m
        assert nx.is_bipartite(G)
        X, Y = nx.algorithms.bipartite.sets(G)
        #print(X)
        assert set(range(n)) == X
        assert set(range(n, n + m)) == Y
        assert edges == len(list(G.edges()))



#    Copyright (C) 2010-2019 by
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    All rights reserved.
#    BSD license.
#
# Author: Aric Hagberg (hagberg@lanl.gov)
"""Current-flow betweenness centrality measures for subsets of nodes."""
import itertools

import networkx as nx
from networkx.algorithms.centrality.flow_matrix import *
from networkx.utils import not_implemented_for, reverse_cuthill_mckee_ordering

__all__ = ['current_flow_betweenness_centrality_subset',
           'edge_current_flow_betweenness_centrality_subset']


@not_implemented_for('directed')
def current_flow_betweenness_centrality_subset(G, sources, targets,
                                               normalized=True,
                                               weight=None,
                                               dtype=float, solver='lu'):
    r"""Compute current-flow betweenness centrality for subsets of nodes.

    Current-flow betweenness centrality uses an electrical current
    model for information spreading in contrast to betweenness
    centrality which uses shortest paths.

    Current-flow betweenness centrality is also known as
    random-walk betweenness centrality [2]_.

    Parameters
    ----------
    G : graph
      A NetworkX graph

    sources: list of nodes
      Nodes to use as sources for current

    targets: list of nodes
      Nodes to use as sinks for current

    normalized : bool, optional (default=True)
      If True the betweenness values are normalized by b=b/(n-1)(n-2) where
      n is the number of nodes in G.

    weight : string or None, optional (default=None)
      Key for edge data used as the edge weight.
      If None, then use 1 as each edge weight.

    dtype: data type (float)
      Default data type for internal matrices.
      Set to np.float32 for lower memory consumption.

    solver: string (default='lu')
       Type of linear solver to use for computing the flow matrix.
       Options are "full" (uses most memory), "lu" (recommended), and
       "cg" (uses least memory).

    Returns
    -------
    nodes : dictionary
       Dictionary of nodes with betweenness centrality as the value.

    See Also
    --------
    approximate_current_flow_betweenness_centrality
    betweenness_centrality
    edge_betweenness_centrality
    edge_current_flow_betweenness_centrality

    Notes
    -----
    Current-flow betweenness can be computed in $O(I(n-1)+mn \log n)$
    time [1]_, where $I(n-1)$ is the time needed to compute the
    inverse Laplacian.  For a full matrix this is $O(n^3)$ but using
    sparse methods you can achieve $O(nm{\sqrt k})$ where $k$ is the
    Laplacian matrix condition number.

    The space required is $O(nw)$ where $w$ is the width of the sparse
    Laplacian matrix.  Worse case is $w=n$ for $O(n^2)$.

    If the edges have a 'weight' attribute they will be used as
    weights in this algorithm.  Unspecified weights are set to 1.

    References
    ----------
    .. [1] Centrality Measures Based on Current Flow.
       Ulrik Brandes and Daniel Fleischer,
       Proc. 22nd Symp. Theoretical Aspects of Computer Science (STACS '05).
       LNCS 3404, pp. 533-544. Springer-Verlag, 2005.
       http://algo.uni-konstanz.de/publications/bf-cmbcf-05.pdf

    .. [2] A measure of betweenness centrality based on random walks,
       M. E. J. Newman, Social Networks 27, 39-54 (2005).
    """
    from networkx.utils import reverse_cuthill_mckee_ordering
    try:
        import numpy as np
    except ImportError:
        raise ImportError('current_flow_betweenness_centrality requires NumPy ',
                          'http://scipy.org/')
    try:
        import scipy
    except ImportError:
        raise ImportError('current_flow_betweenness_centrality requires SciPy ',
                          'http://scipy.org/')
    if not nx.is_connected(G):
        raise nx.NetworkXError("Graph not connected.")
    n = G.number_of_nodes()
    ordering = list(reverse_cuthill_mckee_ordering(G))
    # make a copy with integer labels according to rcm ordering
    # this could be done without a copy if we really wanted to
    mapping = dict(zip(ordering, range(n)))
    H = nx.relabel_nodes(G, mapping)
    betweenness = dict.fromkeys(H, 0.0)  # b[v]=0 for v in H
    for row, (s, t) in flow_matrix_row(H, weight=weight, dtype=dtype,
                                       solver=solver):
        for ss in sources:
            i = mapping[ss]
            for tt in targets:
                j = mapping[tt]
                betweenness[s] += 0.5 * np.abs(row[i] - row[j])
                betweenness[t] += 0.5 * np.abs(row[i] - row[j])
    if normalized:
        nb = (n - 1.0) * (n - 2.0)  # normalization factor
    else:
        nb = 2.0
    for v in H:
        betweenness[v] = betweenness[v] / nb + 1.0 / (2 - n)
    return dict((ordering[k], v) for k, v in betweenness.items())


@not_implemented_for('directed')
def edge_current_flow_betweenness_centrality_subset(G, sources, targets,
                                                    normalized=True,
                                                    weight=None,
                                                    dtype=float, solver='lu'):
    r"""Compute current-flow betweenness centrality for edges using subsets
    of nodes.

    Current-flow betweenness centrality uses an electrical current
    model for information spreading in contrast to betweenness
    centrality which uses shortest paths.

    Current-flow betweenness centrality is also known as
    random-walk betweenness centrality [2]_.

    Parameters
    ----------
    G : graph
      A NetworkX graph

    sources: list of nodes
      Nodes to use as sources for current

    targets: list of nodes
      Nodes to use as sinks for current

    normalized : bool, optional (default=True)
      If True the betweenness values are normalized by b=b/(n-1)(n-2) where
      n is the number of nodes in G.

    weight : string or None, optional (default=None)
      Key for edge data used as the edge weight.
      If None, then use 1 as each edge weight.

    dtype: data type (float)
      Default data type for internal matrices.
      Set to np.float32 for lower memory consumption.

    solver: string (default='lu')
       Type of linear solver to use for computing the flow matrix.
       Options are "full" (uses most memory), "lu" (recommended), and
       "cg" (uses least memory).

    Returns
    -------
    nodes : dict
       Dictionary of edge tuples with betweenness centrality as the value.

    See Also
    --------
    betweenness_centrality
    edge_betweenness_centrality
    current_flow_betweenness_centrality

    Notes
    -----
    Current-flow betweenness can be computed in $O(I(n-1)+mn \log n)$
    time [1]_, where $I(n-1)$ is the time needed to compute the
    inverse Laplacian.  For a full matrix this is $O(n^3)$ but using
    sparse methods you can achieve $O(nm{\sqrt k})$ where $k$ is the
    Laplacian matrix condition number.

    The space required is $O(nw)$ where $w$ is the width of the sparse
    Laplacian matrix.  Worse case is $w=n$ for $O(n^2)$.

    If the edges have a 'weight' attribute they will be used as
    weights in this algorithm.  Unspecified weights are set to 1.

    References
    ----------
    .. [1] Centrality Measures Based on Current Flow.
       Ulrik Brandes and Daniel Fleischer,
       Proc. 22nd Symp. Theoretical Aspects of Computer Science (STACS '05).
       LNCS 3404, pp. 533-544. Springer-Verlag, 2005.
       http://algo.uni-konstanz.de/publications/bf-cmbcf-05.pdf

    .. [2] A measure of betweenness centrality based on random walks,
       M. E. J. Newman, Social Networks 27, 39-54 (2005).
    """
    try:
        import numpy as np
    except ImportError:
        raise ImportError('current_flow_betweenness_centrality requires NumPy ',
                          'http://scipy.org/')
    try:
        import scipy
    except ImportError:
        raise ImportError('current_flow_betweenness_centrality requires SciPy ',
                          'http://scipy.org/')
    if not nx.is_connected(G):
        raise nx.NetworkXError("Graph not connected.")
    n = G.number_of_nodes()
    ordering = list(reverse_cuthill_mckee_ordering(G))
    # make a copy with integer labels according to rcm ordering
    # this could be done without a copy if we really wanted to
    mapping = dict(zip(ordering, range(n)))
    H = nx.relabel_nodes(G, mapping)
    edges = (tuple(sorted((u, v))) for u, v in H.edges())
    betweenness = dict.fromkeys(edges, 0.0)
    if normalized:
        nb = (n - 1.0) * (n - 2.0)  # normalization factor
    else:
        nb = 2.0
    for row, (e) in flow_matrix_row(H, weight=weight, dtype=dtype,
                                    solver=solver):
        for ss in sources:
            i = mapping[ss]
            for tt in targets:
                j = mapping[tt]
                betweenness[e] += 0.5 * np.abs(row[i] - row[j])
        betweenness[e] /= nb
    return dict(((ordering[s], ordering[t]), v)
                for (s, t), v in betweenness.items())


# fixture for pytest
def setup_module(module):
    import pytest
    numpy = pytest.importorskip('numpy')
    scipy = pytest.importorskip('scipy')



"""
    Unit tests for VoteRank.
"""


import networkx as nx


class TestVoteRankCentrality:
    def test_voterank_centrality_1(self):
        G = nx.Graph()
        G.add_edges_from([(7, 8), (7, 5), (7, 9), (5, 0), (0, 1), (0, 2),
                          (0, 3), (0, 4), (1, 6), (2, 6), (3, 6), (4, 6)])
        assert [0, 7, 6] == nx.voterank(G)

    def test_voterank_centrality_2(self):
        G = nx.florentine_families_graph()
        d = nx.voterank(G, 4)
        exact = ['Medici', 'Strozzi', 'Guadagni', 'Castellani']
        assert exact == d



import pytest
from itertools import chain, combinations, product

import networkx as nx

tree_all_pairs_lca = nx.tree_all_pairs_lowest_common_ancestor
all_pairs_lca = nx.all_pairs_lowest_common_ancestor


def get_pair(dictionary, n1, n2):
    if (n1, n2) in dictionary:
        return dictionary[n1, n2]
    else:
        return dictionary[n2, n1]


class TestTreeLCA(object):
    @classmethod
    def setup_class(cls):
        cls.DG = nx.DiGraph()
        edges = [(0, 1), (0, 2), (1, 3), (1, 4), (2, 5), (2, 6)]
        cls.DG.add_edges_from(edges)
        cls.ans = dict(tree_all_pairs_lca(cls.DG, 0))
        gold = dict([((n, n), n) for n in cls.DG])
        gold.update(dict(((0, i), 0) for i in range(1, 7)))
        gold.update({(1, 2): 0,
                     (1, 3): 1,
                     (1, 4): 1,
                     (1, 5): 0,
                     (1, 6): 0,
                     (2, 3): 0,
                     (2, 4): 0,
                     (2, 5): 2,
                     (2, 6): 2,
                     (3, 4): 1,
                     (3, 5): 0,
                     (3, 6): 0,
                     (4, 5): 0,
                     (4, 6): 0,
                     (5, 6): 2})

        cls.gold = gold

    @staticmethod
    def assert_has_same_pairs(d1, d2):
        for (a, b) in ((min(pair), max(pair)) for pair in chain(d1, d2)):
            assert get_pair(d1, a, b) == get_pair(d2, a, b)

    def test_tree_all_pairs_lowest_common_ancestor1(self):
        """Specifying the root is optional."""
        assert dict(tree_all_pairs_lca(self.DG)) == self.ans

    def test_tree_all_pairs_lowest_common_ancestor2(self):
        """Specifying only some pairs gives only those pairs."""
        test_pairs = [(0, 1), (0, 1), (1, 0)]
        ans = dict(tree_all_pairs_lca(self.DG, 0, test_pairs))
        assert (0, 1) in ans and (1, 0) in ans
        assert len(ans) == 2

    def test_tree_all_pairs_lowest_common_ancestor3(self):
        """Specifying no pairs same as specifying all."""
        all_pairs = chain(combinations(self.DG, 2),
                          ((node, node) for node in self.DG))

        ans = dict(tree_all_pairs_lca(self.DG, 0, all_pairs))
        self.assert_has_same_pairs(ans, self.ans)

    def test_tree_all_pairs_lowest_common_ancestor4(self):
        """Gives the right answer."""
        ans = dict(tree_all_pairs_lca(self.DG))
        self.assert_has_same_pairs(self.gold, ans)

    def test_tree_all_pairs_lowest_common_ancestor5(self):
        """Handles invalid input correctly."""
        empty_digraph = tree_all_pairs_lca(nx.DiGraph())
        pytest.raises(nx.NetworkXPointlessConcept, list, empty_digraph)

        bad_pairs_digraph = tree_all_pairs_lca(self.DG, pairs=[(-1, -2)])
        pytest.raises(nx.NodeNotFound, list, bad_pairs_digraph)

    def test_tree_all_pairs_lowest_common_ancestor6(self):
        """Works on subtrees."""
        ans = dict(tree_all_pairs_lca(self.DG, 1))
        gold = dict((pair, lca) for (pair, lca) in self.gold.items()
                    if all(n in (1, 3, 4) for n in pair))
        self.assert_has_same_pairs(gold, ans)

    def test_tree_all_pairs_lowest_common_ancestor7(self):
        """Works on disconnected nodes."""
        G = nx.DiGraph()
        G.add_node(1)
        assert {(1, 1): 1} == dict(tree_all_pairs_lca(G))

        G.add_node(0)
        assert {(1, 1): 1} == dict(tree_all_pairs_lca(G, 1))
        assert {(0, 0): 0} == dict(tree_all_pairs_lca(G, 0))

        pytest.raises(nx.NetworkXError, list, tree_all_pairs_lca(G))

    def test_tree_all_pairs_lowest_common_ancestor8(self):
        """Raises right errors if not a tree."""
        # Cycle
        G = nx.DiGraph([(1, 2), (2, 1)])
        pytest.raises(nx.NetworkXError, list, tree_all_pairs_lca(G))
        # DAG
        G = nx.DiGraph([(0, 2), (1, 2)])
        pytest.raises(nx.NetworkXError, list, tree_all_pairs_lca(G))

    def test_tree_all_pairs_lowest_common_ancestor9(self):
        """Test that pairs works correctly as a generator."""
        pairs = iter([(0, 1), (0, 1), (1, 0)])
        some_pairs = dict(tree_all_pairs_lca(self.DG, 0, pairs))
        assert (0, 1) in some_pairs and (1, 0) in some_pairs
        assert len(some_pairs) == 2

    def test_tree_all_pairs_lowest_common_ancestor10(self):
        """Test that pairs not in the graph raises error."""
        lca = tree_all_pairs_lca(self.DG, 0, [(-1, -1)])
        pytest.raises(nx.NodeNotFound, list, lca)

    def test_tree_all_pairs_lowest_common_ancestor11(self):
        """Test that None as a node in the graph raises an error."""
        G = nx.DiGraph([(None, 3)])
        pytest.raises(nx.NetworkXError, list, tree_all_pairs_lca(G))
        pytest.raises(nx.NodeNotFound, list,
                      tree_all_pairs_lca(self.DG, pairs=G.edges()))

    def test_tree_all_pairs_lowest_common_ancestor12(self):
        """Test that tree routine bails on DAGs."""
        G = nx.DiGraph([(3, 4), (5, 4)])
        pytest.raises(nx.NetworkXError, list, tree_all_pairs_lca(G))

    def test_not_implemented_for(self):
        NNI = nx.NetworkXNotImplemented
        G = nx.Graph([(0, 1)])
        pytest.raises(NNI, tree_all_pairs_lca, G)
        pytest.raises(NNI, all_pairs_lca, G)
        pytest.raises(NNI, nx.lowest_common_ancestor, G, 0, 1)
        G = nx.MultiGraph([(0, 1)])
        pytest.raises(NNI, tree_all_pairs_lca, G)
        pytest.raises(NNI, all_pairs_lca, G)
        pytest.raises(NNI, nx.lowest_common_ancestor, G, 0, 1)
        G = nx.MultiDiGraph([(0, 1)])
        pytest.raises(NNI, tree_all_pairs_lca, G)
        pytest.raises(NNI, all_pairs_lca, G)
        pytest.raises(NNI, nx.lowest_common_ancestor, G, 0, 1)

    def test_tree_all_pairs_lowest_common_ancestor13(self):
        """Test that it works on non-empty trees with no LCAs."""
        G = nx.DiGraph()
        G.add_node(3)
        ans = list(tree_all_pairs_lca(G))
        assert ans == [((3, 3), 3)]


class TestDAGLCA:
    @classmethod
    def setup_class(cls):
        cls.DG = nx.DiGraph()
        nx.add_path(cls.DG, (0, 1, 2, 3))
        nx.add_path(cls.DG, (0, 4, 3))
        nx.add_path(cls.DG, (0, 5, 6, 8, 3))
        nx.add_path(cls.DG, (5, 7, 8))
        cls.DG.add_edge(6, 2)
        cls.DG.add_edge(7, 2)

        cls.root_distance = nx.shortest_path_length(cls.DG, source=0)

        cls.gold = {(1, 1): 1,
                     (1, 2): 1,
                     (1, 3): 1,
                     (1, 4): 0,
                     (1, 5): 0,
                     (1, 6): 0,
                     (1, 7): 0,
                     (1, 8): 0,
                     (2, 2): 2,
                     (2, 3): 2,
                     (2, 4): 0,
                     (2, 5): 5,
                     (2, 6): 6,
                     (2, 7): 7,
                     (2, 8): 7,
                     (3, 3): 8,
                     (3, 4): 4,
                     (3, 5): 5,
                     (3, 6): 6,
                     (3, 7): 7,
                     (3, 8): 8,
                     (4, 4): 4,
                     (4, 5): 0,
                     (4, 6): 0,
                     (4, 7): 0,
                     (4, 8): 0,
                     (5, 5): 5,
                     (5, 6): 5,
                     (5, 7): 5,
                     (5, 8): 5,
                     (6, 6): 6,
                     (6, 7): 5,
                     (6, 8): 6,
                     (7, 7): 7,
                     (7, 8): 7,
                     (8, 8): 8}
        cls.gold.update(((0, n), 0) for n in cls.DG)

    def assert_lca_dicts_same(self, d1, d2, G=None):
        """Checks if d1 and d2 contain the same pairs and
        have a node at the same distance from root for each.
        If G is None use self.DG."""
        if G is None:
            G = self.DG
            root_distance = self.root_distance
        else:
            roots = [n for n, deg in G.in_degree if deg == 0]
            assert(len(roots) == 1)
            root_distance = nx.shortest_path_length(G, source=roots[0])

        for a, b in ((min(pair), max(pair)) for pair in chain(d1, d2)):
            assert (root_distance[get_pair(d1, a, b)] ==
                         root_distance[get_pair(d2, a, b)])

    def test_all_pairs_lowest_common_ancestor1(self):
        """Produces the correct results."""
        self.assert_lca_dicts_same(dict(all_pairs_lca(self.DG)), self.gold)

    def test_all_pairs_lowest_common_ancestor2(self):
        """Produces the correct results when all pairs given."""
        all_pairs = list(product(self.DG.nodes(), self.DG.nodes()))
        ans = all_pairs_lca(self.DG, pairs=all_pairs)
        self.assert_lca_dicts_same(dict(ans), self.gold)

    def test_all_pairs_lowest_common_ancestor3(self):
        """Produces the correct results when all pairs given as a generator."""
        all_pairs = product(self.DG.nodes(), self.DG.nodes())
        ans = all_pairs_lca(self.DG, pairs=all_pairs)
        self.assert_lca_dicts_same(dict(ans), self.gold)

    def test_all_pairs_lowest_common_ancestor4(self):
        """Graph with two roots."""
        G = self.DG.copy()
        G.add_edge(9, 10)
        G.add_edge(9, 4)
        gold = self.gold.copy()
        gold[9, 9] = 9
        gold[9, 10] = 9
        gold[9, 4] = 9
        gold[9, 3] = 9
        gold[10, 4] = 9
        gold[10, 3] = 9
        gold[10, 10] = 10

        testing = dict(all_pairs_lca(G))

        G.add_edge(-1, 9)
        G.add_edge(-1, 0)
        self.assert_lca_dicts_same(testing, gold, G)

    def test_all_pairs_lowest_common_ancestor5(self):
        """Test that pairs not in the graph raises error."""
        pytest.raises(nx.NodeNotFound, all_pairs_lca, self.DG, [(-1, -1)])

    def test_all_pairs_lowest_common_ancestor6(self):
        """Test that pairs with no LCA specified emits nothing."""
        G = self.DG.copy()
        G.add_node(-1)
        gen = all_pairs_lca(G, [(-1, -1), (-1, 0)])
        assert dict(gen) == {(-1, -1): -1}

    def test_all_pairs_lowest_common_ancestor7(self):
        """Test that LCA on null graph bails."""
        pytest.raises(nx.NetworkXPointlessConcept,
                      all_pairs_lca,
                      nx.DiGraph())

    def test_all_pairs_lowest_common_ancestor8(self):
        """Test that LCA on non-dags bails."""
        pytest.raises(nx.NetworkXError, all_pairs_lca,
                      nx.DiGraph([(3, 4), (4, 3)]))

    def test_all_pairs_lowest_common_ancestor9(self):
        """Test that it works on non-empty graphs with no LCAs."""
        G = nx.DiGraph()
        G.add_node(3)
        ans = list(all_pairs_lca(G))
        assert ans == [((3, 3), 3)]

    def test_all_pairs_lowest_common_ancestor10(self):
        """Test that it bails on None as a node."""
        G = nx.DiGraph([(None, 3)])
        pytest.raises(nx.NetworkXError, all_pairs_lca, G)
        pytest.raises(nx.NodeNotFound, all_pairs_lca,
                      self.DG, pairs=G.edges())

    def test_lowest_common_ancestor1(self):
        """Test that the one-pair function works on default."""
        G = nx.DiGraph([(0, 1), (2, 1)])
        sentinel = object()
        assert (nx.lowest_common_ancestor(G, 0, 2, default=sentinel) is
                  sentinel)

    def test_lowest_common_ancestor2(self):
        """Test that the one-pair function works on identity."""
        G = nx.DiGraph()
        G.add_node(3)
        assert nx.lowest_common_ancestor(G, 3, 3) == 3



# disjoint_paths.py - Flow based node and edge disjoint paths.
#
# Copyright 2017-2019 NetworkX developers.
#
# This file is part of NetworkX.
#
# NetworkX is distributed under a BSD license; see LICENSE.txt for more
# information.
#
# Author: Jordi Torrents <jordi.t21@gmail.com>
"""Flow based node and edge disjoint paths."""
import networkx as nx
from networkx.exception import NetworkXNoPath
# Define the default maximum flow function to use for the undelying
# maximum flow computations
from networkx.algorithms.flow import edmonds_karp
from networkx.algorithms.flow import preflow_push
from networkx.algorithms.flow import shortest_augmenting_path
default_flow_func = edmonds_karp
# Functions to build auxiliary data structures.
from networkx.algorithms.flow import build_residual_network
from .utils import build_auxiliary_node_connectivity
from .utils import build_auxiliary_edge_connectivity

try:
    from itertools import filterfalse as _filterfalse
except ImportError:  # Python 2
    def _filterfalse(predicate, iterable):
        # https://docs.python.org/3/library/itertools.html
        # filterfalse(lambda x: x%2, range(10)) --> 0 2 4 6 8
        if predicate is None:
            predicate = bool
        for x in iterable:
            if not predicate(x):
                yield x

__all__ = [
    'edge_disjoint_paths',
    'node_disjoint_paths',
]


def edge_disjoint_paths(G, s, t, flow_func=None, cutoff=None, auxiliary=None,
                        residual=None):
    """Returns the edges disjoint paths between source and target.

    Edge disjoint paths are paths that do not share any edge. The
    number of edge disjoint paths between source and target is equal
    to their edge connectivity.

    Parameters
    ----------
    G : NetworkX graph

    s : node
        Source node for the flow.

    t : node
        Sink node for the flow.

    flow_func : function
        A function for computing the maximum flow among a pair of nodes.
        The function has to accept at least three parameters: a Digraph, 
        a source node, and a target node. And return a residual network 
        that follows NetworkX conventions (see :meth:`maximum_flow` for 
        details). If flow_func is None, the default maximum flow function 
        (:meth:`edmonds_karp`) is used. The choice of the default function
        may change from version to version and should not be relied on.
        Default value: None.

    cutoff : int
        Maximum number of paths to yield. Some of the maximum flow
        algorithms, such as :meth:`edmonds_karp` (the default) and 
        :meth:`shortest_augmenting_path` support the cutoff parameter,
        and will terminate when the flow value reaches or exceeds the
        cutoff. Other algorithms will ignore this parameter.
        Default value: None.

    auxiliary : NetworkX DiGraph
        Auxiliary digraph to compute flow based edge connectivity. It has
        to have a graph attribute called mapping with a dictionary mapping
        node names in G and in the auxiliary digraph. If provided
        it will be reused instead of recreated. Default value: None.

    residual : NetworkX DiGraph
        Residual network to compute maximum flow. If provided it will be
        reused instead of recreated. Default value: None.

    Returns
    -------
    paths : generator
        A generator of edge independent paths.

    Raises
    ------
    NetworkXNoPath : exception
        If there is no path between source and target.

    NetworkXError : exception
        If source or target are not in the graph G.

    See also
    --------
    :meth:`node_disjoint_paths`
    :meth:`edge_connectivity`
    :meth:`maximum_flow`
    :meth:`edmonds_karp`
    :meth:`preflow_push`
    :meth:`shortest_augmenting_path`

    Examples
    --------
    We use in this example the platonic icosahedral graph, which has node
    edge connectivity 5, thus there are 5 edge disjoint paths between any
    pair of nodes.

    >>> G = nx.icosahedral_graph()
    >>> len(list(nx.edge_disjoint_paths(G, 0, 6)))
    5


    If you need to compute edge disjoint paths on several pairs of
    nodes in the same graph, it is recommended that you reuse the
    data structures that NetworkX uses in the computation: the 
    auxiliary digraph for edge connectivity, and the residual
    network for the underlying maximum flow computation.

    Example of how to compute edge disjoint paths among all pairs of
    nodes of the platonic icosahedral graph reusing the data 
    structures.

    >>> import itertools
    >>> # You also have to explicitly import the function for 
    >>> # building the auxiliary digraph from the connectivity package
    >>> from networkx.algorithms.connectivity import (
    ...     build_auxiliary_edge_connectivity)
    >>> H = build_auxiliary_edge_connectivity(G)
    >>> # And the function for building the residual network from the
    >>> # flow package
    >>> from networkx.algorithms.flow import build_residual_network
    >>> # Note that the auxiliary digraph has an edge attribute named capacity
    >>> R = build_residual_network(H, 'capacity')
    >>> result = {n: {} for n in G}
    >>> # Reuse the auxiliary digraph and the residual network by passing them
    >>> # as arguments
    >>> for u, v in itertools.combinations(G, 2):
    ...     k = len(list(nx.edge_disjoint_paths(G, u, v, auxiliary=H, residual=R)))
    ...     result[u][v] = k
    >>> all(result[u][v] == 5 for u, v in itertools.combinations(G, 2))
    True

    You can also use alternative flow algorithms for computing edge disjoint
    paths. For instance, in dense networks the algorithm
    :meth:`shortest_augmenting_path` will usually perform better than
    the default :meth:`edmonds_karp` which is faster for sparse
    networks with highly skewed degree distributions. Alternative flow
    functions have to be explicitly imported from the flow package.

    >>> from networkx.algorithms.flow import shortest_augmenting_path
    >>> len(list(nx.edge_disjoint_paths(G, 0, 6, flow_func=shortest_augmenting_path)))
    5

    Notes
    -----
    This is a flow based implementation of edge disjoint paths. We compute
    the maximum flow between source and target on an auxiliary directed
    network. The saturated edges in the residual network after running the
    maximum flow algorithm correspond to edge disjoint paths between source
    and target in the original network. This function handles both directed
    and undirected graphs, and can use all flow algorithms from NetworkX flow
    package.

    """
    if s not in G:
        raise nx.NetworkXError('node %s not in graph' % s)
    if t not in G:
        raise nx.NetworkXError('node %s not in graph' % t)

    if flow_func is None:
        flow_func = default_flow_func

    if auxiliary is None:
        H = build_auxiliary_edge_connectivity(G)
    else:
        H = auxiliary

    # Maximum possible edge disjoint paths
    possible = min(H.out_degree(s), H.in_degree(t))
    if not possible:
        raise NetworkXNoPath

    if cutoff is None:
        cutoff = possible
    else:
        cutoff = min(cutoff, possible)

    # Compute maximum flow between source and target. Flow functions in
    # NetworkX return a residual network.
    kwargs = dict(capacity='capacity', residual=residual, cutoff=cutoff,
                  value_only=True)
    if flow_func is preflow_push:
        del kwargs['cutoff']
    if flow_func is shortest_augmenting_path:
        kwargs['two_phase'] = True
    R = flow_func(H, s, t, **kwargs)

    if R.graph['flow_value'] == 0:
        raise NetworkXNoPath

    # Saturated edges in the residual network form the edge disjoint paths
    # between source and target
    cutset = [(u, v) for u, v, d in R.edges(data=True)
              if d['capacity'] == d['flow'] and d['flow'] > 0]
    # This is equivalent of what flow.utils.build_flow_dict returns, but
    # only for the nodes with saturated edges and without reporting 0 flows.
    flow_dict = {n: {} for edge in cutset for n in edge}
    for u, v in cutset:
        flow_dict[u][v] = 1

    # Rebuild the edge disjoint paths from the flow dictionary.
    paths_found = 0
    for v in list(flow_dict[s]):
        if paths_found >= cutoff:
            # preflow_push does not support cutoff: we have to
            # keep track of the paths founds and stop at cutoff.
            break
        path = [s]
        if v == t:
            path.append(v)
            yield path
            continue
        u = v
        while u != t:
            path.append(u)
            try:
                u, _ = flow_dict[u].popitem()
            except KeyError:
                break
        else:
            path.append(t)
            yield path
            paths_found += 1


def node_disjoint_paths(G, s, t, flow_func=None, cutoff=None, auxiliary=None,
                        residual=None):
    r"""Computes node disjoint paths between source and target.

    Node dijoint paths are paths that only share their first and last
    nodes. The number of node independent paths between two nodes is
    equal to their local node connectivity.

    Parameters
    ----------
    G : NetworkX graph

    s : node
        Source node.

    t : node
        Target node.

    flow_func : function
        A function for computing the maximum flow among a pair of nodes.
        The function has to accept at least three parameters: a Digraph,
        a source node, and a target node. And return a residual network
        that follows NetworkX conventions (see :meth:`maximum_flow` for
        details). If flow_func is None, the default maximum flow function
        (:meth:`edmonds_karp`) is used. See below for details. The choice
        of the default function may change from version to version and
        should not be relied on. Default value: None.

    cutoff : int
        Maximum number of paths to yield. Some of the maximum flow
        algorithms, such as :meth:`edmonds_karp` (the default) and
        :meth:`shortest_augmenting_path` support the cutoff parameter,
        and will terminate when the flow value reaches or exceeds the
        cutoff. Other algorithms will ignore this parameter.
        Default value: None.

    auxiliary : NetworkX DiGraph
        Auxiliary digraph to compute flow based node connectivity. It has
        to have a graph attribute called mapping with a dictionary mapping
        node names in G and in the auxiliary digraph. If provided
        it will be reused instead of recreated. Default value: None.

    residual : NetworkX DiGraph
        Residual network to compute maximum flow. If provided it will be
        reused instead of recreated. Default value: None.

    Returns
    -------
    paths : generator
        Generator of node disjoint paths.

    Raises
    ------
    NetworkXNoPath : exception
        If there is no path between source and target.

    NetworkXError : exception
        If source or target are not in the graph G.

    Examples
    --------
    We use in this example the platonic icosahedral graph, which has node
    node connectivity 5, thus there are 5 node disjoint paths between any
    pair of non neighbor nodes.

    >>> G = nx.icosahedral_graph()
    >>> len(list(nx.node_disjoint_paths(G, 0, 6)))
    5

    If you need to compute node disjoint paths between several pairs of
    nodes in the same graph, it is recommended that you reuse the
    data structures that NetworkX uses in the computation: the
    auxiliary digraph for node connectivity and node cuts, and the
    residual network for the underlying maximum flow computation.

    Example of how to compute node disjoint paths reusing the data
    structures:

    >>> # You also have to explicitly import the function for 
    >>> # building the auxiliary digraph from the connectivity package
    >>> from networkx.algorithms.connectivity import (
    ...     build_auxiliary_node_connectivity)
    >>> H = build_auxiliary_node_connectivity(G)
    >>> # And the function for building the residual network from the
    >>> # flow package
    >>> from networkx.algorithms.flow import build_residual_network
    >>> # Note that the auxiliary digraph has an edge attribute named capacity
    >>> R = build_residual_network(H, 'capacity')
    >>> # Reuse the auxiliary digraph and the residual network by passing them
    >>> # as arguments
    >>> len(list(nx.node_disjoint_paths(G, 0, 6, auxiliary=H, residual=R)))
    5

    You can also use alternative flow algorithms for computing node disjoint
    paths. For instance, in dense networks the algorithm
    :meth:`shortest_augmenting_path` will usually perform better than
    the default :meth:`edmonds_karp` which is faster for sparse
    networks with highly skewed degree distributions. Alternative flow
    functions have to be explicitly imported from the flow package.

    >>> from networkx.algorithms.flow import shortest_augmenting_path
    >>> len(list(nx.node_disjoint_paths(G, 0, 6, flow_func=shortest_augmenting_path)))
    5

    Notes
    -----
    This is a flow based implementation of node disjoint paths. We compute
    the maximum flow between source and target on an auxiliary directed
    network. The saturated edges in the residual network after running the
    maximum flow algorithm correspond to node disjoint paths between source
    and target in the original network. This function handles both directed
    and undirected graphs, and can use all flow algorithms from NetworkX flow
    package.

    See also
    --------
    :meth:`edge_disjoint_paths`
    :meth:`node_connectivity`
    :meth:`maximum_flow`
    :meth:`edmonds_karp`
    :meth:`preflow_push`
    :meth:`shortest_augmenting_path`

    """
    if s not in G:
        raise nx.NetworkXError('node %s not in graph' % s)
    if t not in G:
        raise nx.NetworkXError('node %s not in graph' % t)

    if auxiliary is None:
        H = build_auxiliary_node_connectivity(G)
    else:
        H = auxiliary

    mapping = H.graph.get('mapping', None)
    if mapping is None:
        raise nx.NetworkXError('Invalid auxiliary digraph.')

    # Maximum possible edge disjoint paths
    possible = min(H.out_degree('%sB' % mapping[s]),
                   H.in_degree('%sA' % mapping[t]))
    if not possible:
        raise NetworkXNoPath

    if cutoff is None:
        cutoff = possible
    else:
        cutoff = min(cutoff, possible)

    kwargs = dict(flow_func=flow_func, residual=residual, auxiliary=H,
                  cutoff=cutoff)

    # The edge disjoint paths in the auxiliary digraph correspond to the node
    # disjoint paths in the original graph.
    paths_edges = edge_disjoint_paths(H, '%sB' % mapping[s], '%sA' % mapping[t],
                                      **kwargs)
    for path in paths_edges:
        # Each node in the original graph maps to two nodes in auxiliary graph
        yield list(_unique_everseen(H.nodes[node]['id'] for node in path))


def _unique_everseen(iterable):
    # Adapted from https://docs.python.org/3/library/itertools.html examples
    "List unique elements, preserving order. Remember all elements ever seen."
    # unique_everseen('AAAABBBCCDAABBB') --> A B C D
    seen = set()
    seen_add = seen.add
    for element in _filterfalse(seen.__contains__, iterable):
        seen_add(element)
        yield element



#    Copyright (C) 2017 by
#    Romain Fontugne <romain@iij.ad.jp>
#    All rights reserved.
#    BSD license.
#
# Author:  Romain Fontugne (romain@iij.ad.jp)
"""Functions for estimating the small-world-ness of graphs.

A small world network is characterized by a small average shortest path length,
and a large clustering coefficient.

Small-worldness is commonly measured with the coefficient sigma or omega.

Both coefficients compare the average clustering coefficient and shortest path
length of a given graph against the same quantities for an equivalent random
or lattice graph.

For more information, see the Wikipedia article on small-world network [1]_.

.. [1] Small-world network:: https://en.wikipedia.org/wiki/Small-world_network

"""
import networkx as nx
from networkx.utils import not_implemented_for
from networkx.utils import py_random_state

__all__ = ['random_reference', 'lattice_reference', 'sigma', 'omega']


@py_random_state(3)
@not_implemented_for('directed')
@not_implemented_for('multigraph')
def random_reference(G, niter=1, connectivity=True, seed=None):
    """Compute a random graph by swapping edges of a given graph.

    Parameters
    ----------
    G : graph
        An undirected graph with 4 or more nodes.

    niter : integer (optional, default=1)
        An edge is rewired approximately `niter` times.

    connectivity : boolean (optional, default=True)
        When True, ensure connectivity for the randomized graph.

    seed : integer, random_state, or None (default)
        Indicator of random number generation state.
        See :ref:`Randomness<randomness>`.

    Returns
    -------
    G : graph
        The randomized graph.

    Notes
    -----
    The implementation is adapted from the algorithm by Maslov and Sneppen
    (2002) [1]_.

    References
    ----------
    .. [1] Maslov, Sergei, and Kim Sneppen.
           "Specificity and stability in topology of protein networks."
           Science 296.5569 (2002): 910-913.
    """
    if G.is_directed():
        msg = "random_reference() not defined for directed graphs."
        raise nx.NetworkXError(msg)
    if len(G) < 4:
        raise nx.NetworkXError("Graph has less than four nodes.")

    from networkx.utils import cumulative_distribution, discrete_sequence
    local_conn = nx.connectivity.local_edge_connectivity

    G = G.copy()
    keys, degrees = zip(*G.degree())  # keys, degree
    cdf = nx.utils.cumulative_distribution(degrees)  # cdf of degree
    nnodes = len(G)
    nedges = nx.number_of_edges(G)
    niter = niter*nedges
    ntries = int(nnodes*nedges/(nnodes*(nnodes-1)/2))
    swapcount = 0

    for i in range(niter):
        n = 0
        while n < ntries:
            # pick two random edges without creating edge list
            # choose source node indices from discrete distribution
            (ai, ci) = discrete_sequence(2, cdistribution=cdf, seed=seed)
            if ai == ci:
                continue  # same source, skip
            a = keys[ai]  # convert index to label
            c = keys[ci]
            # choose target uniformly from neighbors
            b = seed.choice(list(G.neighbors(a)))
            d = seed.choice(list(G.neighbors(c)))
            bi = keys.index(b)
            di = keys.index(d)
            if b in [a, c, d] or d in [a, b, c]:
                continue  # all vertices should be different

            # don't create parallel edges
            if (d not in G[a]) and (b not in G[c]):
                G.add_edge(a, d)
                G.add_edge(c, b)
                G.remove_edge(a, b)
                G.remove_edge(c, d)

                # Check if the graph is still connected
                if connectivity and local_conn(G, a, b) == 0:
                    # Not connected, revert the swap
                    G.remove_edge(a, d)
                    G.remove_edge(c, b)
                    G.add_edge(a, b)
                    G.add_edge(c, d)
                else:
                    swapcount += 1
                    break
            n += 1
    return G


@py_random_state(4)
@not_implemented_for('directed')
@not_implemented_for('multigraph')
def lattice_reference(G, niter=1, D=None, connectivity=True, seed=None):
    """Latticize the given graph by swapping edges.

    Parameters
    ----------
    G : graph
        An undirected graph with 4 or more nodes.

    niter : integer (optional, default=1)
        An edge is rewired approximatively niter times.

    D : numpy.array (optional, default=None)
        Distance to the diagonal matrix.

    connectivity : boolean (optional, default=True)
        Ensure connectivity for the latticized graph when set to True.

    seed : integer, random_state, or None (default)
        Indicator of random number generation state.
        See :ref:`Randomness<randomness>`.

    Returns
    -------
    G : graph
        The latticized graph.

    Notes
    -----
    The implementation is adapted from the algorithm by Sporns et al. [1]_.
    which is inspired from the original work by Maslov and Sneppen(2002) [2]_.

    References
    ----------
    .. [1] Sporns, Olaf, and Jonathan D. Zwi.
       "The small world of the cerebral cortex."
       Neuroinformatics 2.2 (2004): 145-162.
    .. [2] Maslov, Sergei, and Kim Sneppen.
       "Specificity and stability in topology of protein networks."
       Science 296.5569 (2002): 910-913.
    """
    import numpy as np
    from networkx.utils import cumulative_distribution, discrete_sequence
    local_conn = nx.connectivity.local_edge_connectivity

    if G.is_directed():
        msg = "lattice_reference() not defined for directed graphs."
        raise nx.NetworkXError(msg)
    if len(G) < 4:
        raise nx.NetworkXError("Graph has less than four nodes.")
    # Instead of choosing uniformly at random from a generated edge list,
    # this algorithm chooses nonuniformly from the set of nodes with
    # probability weighted by degree.
    G = G.copy()
    keys, degrees = zip(*G.degree())  # keys, degree
    cdf = cumulative_distribution(degrees)  # cdf of degree

    nnodes = len(G)
    nedges = nx.number_of_edges(G)
    if D is None:
        D = np.zeros((nnodes, nnodes))
        un = np.arange(1, nnodes)
        um = np.arange(nnodes - 1, 0, -1)
        u = np.append((0,), np.where(un < um, un, um))

        for v in range(int(np.ceil(nnodes / 2))):
            D[nnodes - v - 1, :] = np.append(u[v + 1:], u[:v + 1])
            D[v, :] = D[nnodes - v - 1, :][::-1]

    niter = niter*nedges
    ntries = int(nnodes * nedges / (nnodes * (nnodes - 1) / 2))
    swapcount = 0

    for i in range(niter):
        n = 0
        while n < ntries:
            # pick two random edges without creating edge list
            # choose source node indices from discrete distribution
            (ai, ci) = discrete_sequence(2, cdistribution=cdf, seed=seed)
            if ai == ci:
                continue  # same source, skip
            a = keys[ai]  # convert index to label
            c = keys[ci]
            # choose target uniformly from neighbors
            b = seed.choice(list(G.neighbors(a)))
            d = seed.choice(list(G.neighbors(c)))
            bi = keys.index(b)
            di = keys.index(d)

            if b in [a, c, d] or d in [a, b, c]:
                continue  # all vertices should be different

            # don't create parallel edges
            if (d not in G[a]) and (b not in G[c]):
                if D[ai, bi] + D[ci, di] >= D[ai, ci] + D[bi, di]:
                    # only swap if we get closer to the diagonal
                    G.add_edge(a, d)
                    G.add_edge(c, b)
                    G.remove_edge(a, b)
                    G.remove_edge(c, d)

                    # Check if the graph is still connected
                    if connectivity and local_conn(G, a, b) == 0:
                        # Not connected, revert the swap
                        G.remove_edge(a, d)
                        G.remove_edge(c, b)
                        G.add_edge(a, b)
                        G.add_edge(c, d)
                    else:
                        swapcount += 1
                        break
            n += 1

    return G


@py_random_state(3)
@not_implemented_for('directed')
@not_implemented_for('multigraph')
def sigma(G, niter=100, nrand=10, seed=None):
    """Returns the small-world coefficient (sigma) of the given graph.

    The small-world coefficient is defined as:
    sigma = C/Cr / L/Lr
    where C and L are respectively the average clustering coefficient and
    average shortest path length of G. Cr and Lr are respectively the average
    clustering coefficient and average shortest path length of an equivalent
    random graph.

    A graph is commonly classified as small-world if sigma>1.

    Parameters
    ----------
    G : NetworkX graph
        An undirected graph.
    niter : integer (optional, default=100)
        Approximate number of rewiring per edge to compute the equivalent
        random graph.
    nrand : integer (optional, default=10)
        Number of random graphs generated to compute the average clustering
        coefficient (Cr) and average shortest path length (Lr).
    seed : integer, random_state, or None (default)
        Indicator of random number generation state.
        See :ref:`Randomness<randomness>`.

    Returns
    -------
    sigma : float
        The small-world coefficient of G.

    Notes
    -----
    The implementation is adapted from Humphries et al. [1]_ [2]_.

    References
    ----------
    .. [1] The brainstem reticular formation is a small-world, not scale-free,
           network M. D. Humphries, K. Gurney and T. J. Prescott,
           Proc. Roy. Soc. B 2006 273, 503-511, doi:10.1098/rspb.2005.3354.
    .. [2] Humphries and Gurney (2008).
           "Network 'Small-World-Ness': A Quantitative Method for Determining
           Canonical Network Equivalence".
           PLoS One. 3 (4). PMID 18446219. doi:10.1371/journal.pone.0002051.
    """
    import numpy as np

    # Compute the mean clustering coefficient and average shortest path length
    # for an equivalent random graph
    randMetrics = {"C": [], "L": []}
    for i in range(nrand):
        Gr = random_reference(G, niter=niter, seed=seed)
        randMetrics["C"].append(nx.transitivity(Gr))
        randMetrics["L"].append(nx.average_shortest_path_length(Gr))

    C = nx.transitivity(G)
    L = nx.average_shortest_path_length(G)
    Cr = np.mean(randMetrics["C"])
    Lr = np.mean(randMetrics["L"])

    sigma = (C / Cr) / (L / Lr)

    return sigma


@py_random_state(3)
@not_implemented_for('directed')
@not_implemented_for('multigraph')
def omega(G, niter=100, nrand=10, seed=None):
    """Returns the small-world coefficient (omega) of a graph

    The small-world coefficient of a graph G is:

    omega = Lr/L - C/Cl

    where C and L are respectively the average clustering coefficient and
    average shortest path length of G. Lr is the average shortest path length
    of an equivalent random graph and Cl is the average clustering coefficient
    of an equivalent lattice graph.

    The small-world coefficient (omega) ranges between -1 and 1. Values close
    to 0 means the G features small-world characteristics. Values close to -1
    means G has a lattice shape whereas values close to 1 means G is a random
    graph.

    Parameters
    ----------
    G : NetworkX graph
        An undirected graph.

    niter: integer (optional, default=100)
        Approximate number of rewiring per edge to compute the equivalent
        random graph.

    nrand: integer (optional, default=10)
        Number of random graphs generated to compute the average clustering
        coefficient (Cr) and average shortest path length (Lr).

    seed : integer, random_state, or None (default)
        Indicator of random number generation state.
        See :ref:`Randomness<randomness>`.


    Returns
    -------
    omega : float
        The small-work coefficient (omega)

    Notes
    -----
    The implementation is adapted from the algorithm by Telesford et al. [1]_.

    References
    ----------
    .. [1] Telesford, Joyce, Hayasaka, Burdette, and Laurienti (2011).
           "The Ubiquity of Small-World Networks".
           Brain Connectivity. 1 (0038): 367-75.  PMC 3604768. PMID 22432451.
           doi:10.1089/brain.2011.0038.
    """
    import numpy as np

    # Compute the mean clustering coefficient and average shortest path length
    # for an equivalent random graph
    randMetrics = {"C": [], "L": []}
    for i in range(nrand):
        Gr = random_reference(G, niter=niter, seed=seed)
        Gl = lattice_reference(G, niter=niter, seed=seed)
        randMetrics["C"].append(nx.transitivity(Gl))
        randMetrics["L"].append(nx.average_shortest_path_length(Gr))

    C = nx.transitivity(G)
    L = nx.average_shortest_path_length(G)
    Cl = np.mean(randMetrics["C"])
    Lr = np.mean(randMetrics["L"])

    omega = (Lr / L) - (C / Cl)

    return omega



import networkx as nx
from networkx.algorithms.approximation import average_clustering

# This approximation has to be be exact in regular graphs
# with no triangles or with all possible triangles.


def test_petersen():
    # Actual coefficient is 0
    G = nx.petersen_graph()
    assert (average_clustering(G, trials=int(len(G) / 2)) ==
                 nx.average_clustering(G))


def test_petersen_seed():
    # Actual coefficient is 0
    G = nx.petersen_graph()
    assert (average_clustering(G, trials=int(len(G) / 2), seed=1) ==
                 nx.average_clustering(G))


def test_tetrahedral():
    # Actual coefficient is 1
    G = nx.tetrahedral_graph()
    assert (average_clustering(G, trials=int(len(G) / 2)) ==
                 nx.average_clustering(G))


def test_dodecahedral():
    # Actual coefficient is 0
    G = nx.dodecahedral_graph()
    assert (average_clustering(G, trials=int(len(G) / 2)) ==
                 nx.average_clustering(G))


def test_empty():
    G = nx.empty_graph(5)
    assert average_clustering(G, trials=int(len(G) / 2)) == 0


def test_complete():
    G = nx.complete_graph(5)
    assert average_clustering(G, trials=int(len(G) / 2)) == 1
    G = nx.complete_graph(7)
    assert average_clustering(G, trials=int(len(G) / 2)) == 1



import json
import pytest
import networkx as nx
from networkx.readwrite.json_graph import *


class TestCytoscape:

    def test_graph(self):
        G = nx.path_graph(4)
        H = cytoscape_graph(cytoscape_data(G))
        nx.is_isomorphic(G, H)

    def test_graph_attributes(self):
        G = nx.path_graph(4)
        G.add_node(1, color='red')
        G.add_edge(1, 2, width=7)
        G.graph['foo'] = 'bar'
        G.graph[1] = 'one'
        G.add_node(3, name="node", id="123")

        H = cytoscape_graph(cytoscape_data(G))
        assert H.graph['foo'] == 'bar'
        assert H.nodes[1]['color'] == 'red'
        assert H[1][2]['width'] == 7
        assert H.nodes[3]['name'] == 'node'
        assert H.nodes[3]['id'] == '123'

        d = json.dumps(cytoscape_data(G))
        H = cytoscape_graph(json.loads(d))
        assert H.graph['foo'] == 'bar'
        assert H.graph[1] == 'one'
        assert H.nodes[1]['color'] == 'red'
        assert H[1][2]['width'] == 7
        assert H.nodes[3]['name'] == 'node'
        assert H.nodes[3]['id'] == '123'

    def test_digraph(self):
        G = nx.DiGraph()
        nx.add_path(G, [1, 2, 3])
        H = cytoscape_graph(cytoscape_data(G))
        assert H.is_directed()
        nx.is_isomorphic(G, H)

    def test_multidigraph(self):
        G = nx.MultiDiGraph()
        nx.add_path(G, [1, 2, 3])
        H = cytoscape_graph(cytoscape_data(G))
        assert H.is_directed()
        assert H.is_multigraph()

    def test_multigraph(self):
        G = nx.MultiGraph()
        G.add_edge(1, 2, key='first')
        G.add_edge(1, 2, key='second', color='blue')
        H = cytoscape_graph(cytoscape_data(G))
        assert nx.is_isomorphic(G, H)
        assert H[1][2]['second']['color'] == 'blue'

    def test_exception(self):
        with pytest.raises(nx.NetworkXError):
            G = nx.MultiDiGraph()
            attrs = dict(name='node', ident='node')
            cytoscape_data(G, attrs)



import os
import inspect
import networkx as nx

print("Run this script from the doc/ directory of the repository")
funcs = inspect.getmembers(nx, inspect.isfunction)

for n,f in funcs:
    #print(n + ": "+str(f))
    cmd = "find . -name *\."+n+".rst -print"
    #print(cmd)
    result=os.popen(cmd).read()
    #print(result)

    old_names = ('find_cores',
                 'test',
                 'edge_betweenness',
                 'betweenness_centrality_source',
                 'write_graphml_lxml',
                 'write_graphml_xml',
                 'adj_matrix',
                 'project',
                 'fruchterman_reingold_layout',
                 'node_degree_xy',
                 'node_attribute_xy',
                 'find_cliques_recursive',
                 'recursive_simple_cycles',
                 )

    if len(result) == 0 and n not in old_names:
        print("Missing file from docs:  ", n)

print("Done finding functions that are missing from the docs")



import networkx as nx
from networkx.algorithms import bipartite
from networkx.testing import almost_equal

class TestBipartiteCentrality(object):

    @classmethod
    def setup_class(cls):
        cls.P4 = nx.path_graph(4)
        cls.K3 = nx.complete_bipartite_graph(3, 3)
        cls.C4 = nx.cycle_graph(4)
        cls.davis = nx.davis_southern_women_graph()
        cls.top_nodes = [n for n, d in cls.davis.nodes(data=True)
                          if d['bipartite'] == 0]

    def test_degree_centrality(self):
        d = bipartite.degree_centrality(self.P4, [1, 3])
        answer = {0: 0.5, 1: 1.0,  2: 1.0, 3:  0.5}
        assert d == answer
        d = bipartite.degree_centrality(self.K3, [0, 1, 2])
        answer = {0: 1.0, 1: 1.0, 2: 1.0, 3: 1.0, 4: 1.0, 5: 1.0}
        assert d == answer
        d = bipartite.degree_centrality(self.C4, [0, 2])
        answer = {0: 1.0, 1: 1.0, 2: 1.0, 3: 1.0}
        assert d == answer

    def test_betweenness_centrality(self):
        c = bipartite.betweenness_centrality(self.P4, [1, 3])
        answer = {0: 0.0, 1: 1.0, 2: 1.0, 3: 0.0}
        assert c == answer
        c = bipartite.betweenness_centrality(self.K3, [0, 1, 2])
        answer = {0: 0.125, 1: 0.125, 2: 0.125, 3: 0.125, 4: 0.125, 5: 0.125}
        assert c == answer
        c = bipartite.betweenness_centrality(self.C4, [0, 2])
        answer = {0: 0.25, 1: 0.25, 2: 0.25, 3: 0.25}
        assert c == answer

    def test_closeness_centrality(self):
        c = bipartite.closeness_centrality(self.P4, [1, 3])
        answer = {0: 2.0 / 3, 1: 1.0, 2: 1.0, 3: 2.0 / 3}
        assert c == answer
        c = bipartite.closeness_centrality(self.K3, [0, 1, 2])
        answer = {0: 1.0, 1: 1.0, 2: 1.0, 3: 1.0, 4: 1.0, 5: 1.0}
        assert c == answer
        c = bipartite.closeness_centrality(self.C4, [0, 2])
        answer = {0: 1.0, 1: 1.0, 2: 1.0, 3: 1.0}
        assert c == answer
        G = nx.Graph()
        G.add_node(0)
        G.add_node(1)
        c = bipartite.closeness_centrality(G, [0])
        assert c == {1: 0.0}
        c = bipartite.closeness_centrality(G, [1])
        assert c == {1: 0.0}

    def test_davis_degree_centrality(self):
        G = self.davis
        deg = bipartite.degree_centrality(G, self.top_nodes)
        answer = {'E8': 0.78,
                  'E9': 0.67,
                  'E7': 0.56,
                  'Nora Fayette': 0.57,
                  'Evelyn Jefferson': 0.57,
                  'Theresa Anderson': 0.57,
                  'E6': 0.44,
                  'Sylvia Avondale': 0.50,
                  'Laura Mandeville': 0.50,
                  'Brenda Rogers': 0.50,
                  'Katherina Rogers': 0.43,
                  'E5': 0.44,
                  'Helen Lloyd': 0.36,
                  'E3': 0.33,
                  'Ruth DeSand': 0.29,
                  'Verne Sanderson': 0.29,
                  'E12': 0.33,
                  'Myra Liddel': 0.29,
                  'E11': 0.22,
                  'Eleanor Nye': 0.29,
                  'Frances Anderson': 0.29,
                  'Pearl Oglethorpe': 0.21,
                  'E4': 0.22,
                  'Charlotte McDowd': 0.29,
                  'E10': 0.28,
                  'Olivia Carleton': 0.14,
                  'Flora Price': 0.14,
                  'E2': 0.17,
                  'E1': 0.17,
                  'Dorothy Murchison': 0.14,
                  'E13': 0.17,
                  'E14': 0.17}
        for node, value in answer.items():
            assert almost_equal(value, deg[node], places=2)

    def test_davis_betweenness_centrality(self):
        G = self.davis
        bet = bipartite.betweenness_centrality(G, self.top_nodes)
        answer = {'E8': 0.24,
                  'E9': 0.23,
                  'E7': 0.13,
                  'Nora Fayette': 0.11,
                  'Evelyn Jefferson': 0.10,
                  'Theresa Anderson': 0.09,
                  'E6': 0.07,
                  'Sylvia Avondale': 0.07,
                  'Laura Mandeville': 0.05,
                  'Brenda Rogers': 0.05,
                  'Katherina Rogers': 0.05,
                  'E5': 0.04,
                  'Helen Lloyd': 0.04,
                  'E3': 0.02,
                  'Ruth DeSand': 0.02,
                  'Verne Sanderson': 0.02,
                  'E12': 0.02,
                  'Myra Liddel': 0.02,
                  'E11': 0.02,
                  'Eleanor Nye': 0.01,
                  'Frances Anderson': 0.01,
                  'Pearl Oglethorpe': 0.01,
                  'E4': 0.01,
                  'Charlotte McDowd': 0.01,
                  'E10': 0.01,
                  'Olivia Carleton': 0.01,
                  'Flora Price': 0.01,
                  'E2': 0.00,
                  'E1': 0.00,
                  'Dorothy Murchison': 0.00,
                  'E13': 0.00,
                  'E14': 0.00}
        for node, value in answer.items():
            assert almost_equal(value, bet[node], places=2)

    def test_davis_closeness_centrality(self):
        G = self.davis
        clos = bipartite.closeness_centrality(G, self.top_nodes)
        answer = {'E8': 0.85,
                  'E9': 0.79,
                  'E7': 0.73,
                  'Nora Fayette': 0.80,
                  'Evelyn Jefferson': 0.80,
                  'Theresa Anderson': 0.80,
                  'E6': 0.69,
                  'Sylvia Avondale': 0.77,
                  'Laura Mandeville': 0.73,
                  'Brenda Rogers': 0.73,
                  'Katherina Rogers': 0.73,
                  'E5': 0.59,
                  'Helen Lloyd': 0.73,
                  'E3': 0.56,
                  'Ruth DeSand': 0.71,
                  'Verne Sanderson': 0.71,
                  'E12': 0.56,
                  'Myra Liddel': 0.69,
                  'E11': 0.54,
                  'Eleanor Nye': 0.67,
                  'Frances Anderson': 0.67,
                  'Pearl Oglethorpe': 0.67,
                  'E4': 0.54,
                  'Charlotte McDowd': 0.60,
                  'E10': 0.55,
                  'Olivia Carleton': 0.59,
                  'Flora Price': 0.59,
                  'E2': 0.52,
                  'E1': 0.52,
                  'Dorothy Murchison': 0.65,
                  'E13': 0.52,
                  'E14': 0.52}
        for node, value in answer.items():
            assert almost_equal(value, clos[node], places=2)



#!/usr/bin/env python
import pytest
import networkx as nx


class TestMCS:

    @classmethod
    def setup_class(cls):
        # simple graph
        connected_chordal_G = nx.Graph()
        connected_chordal_G.add_edges_from([(1, 2), (1, 3), (2, 3), (2, 4),
                                            (3, 4), (3, 5), (3, 6), (4, 5),
                                            (4, 6), (5, 6)])
        cls.connected_chordal_G = connected_chordal_G

        chordal_G = nx.Graph()
        chordal_G.add_edges_from([(1, 2), (1, 3), (2, 3), (2, 4), (3, 4),
                                  (3, 5), (3, 6), (4, 5), (4, 6), (5, 6),
                                  (7, 8)])
        chordal_G.add_node(9)
        cls.chordal_G = chordal_G

        non_chordal_G = nx.Graph()
        non_chordal_G.add_edges_from([(1, 2), (1, 3), (2, 4), (2, 5),
                                      (3, 4), (3, 5)])
        cls.non_chordal_G = non_chordal_G

    def test_is_chordal(self):
        assert not nx.is_chordal(self.non_chordal_G)
        assert nx.is_chordal(self.chordal_G)
        assert nx.is_chordal(self.connected_chordal_G)
        assert nx.is_chordal(nx.complete_graph(3))
        assert nx.is_chordal(nx.cycle_graph(3))
        assert not nx.is_chordal(nx.cycle_graph(5))

    def test_induced_nodes(self):
        G = nx.generators.classic.path_graph(10)
        Induced_nodes = nx.find_induced_nodes(G, 1, 9, 2)
        assert Induced_nodes == set([1, 2, 3, 4, 5, 6, 7, 8, 9])
        pytest.raises(nx.NetworkXTreewidthBoundExceeded,
                      nx.find_induced_nodes, G, 1, 9, 1)
        Induced_nodes = nx.find_induced_nodes(self.chordal_G, 1, 6)
        assert Induced_nodes == set([1, 2, 4, 6])
        pytest.raises(nx.NetworkXError,
                      nx.find_induced_nodes, self.non_chordal_G, 1, 5)

    def test_chordal_find_cliques(self):
        cliques = set([frozenset([9]), frozenset([7, 8]), frozenset([1, 2, 3]),
                       frozenset([2, 3, 4]), frozenset([3, 4, 5, 6])])
        assert nx.chordal_graph_cliques(self.chordal_G) == cliques

    def test_chordal_find_cliques_path(self):
        G = nx.path_graph(10)
        cliqueset = nx.chordal_graph_cliques(G)
        for (u, v) in G.edges():
            assert (frozenset([u, v]) in cliqueset
                        or frozenset([v, u]) in cliqueset)

    def test_chordal_find_cliquesCC(self):
        cliques = set([frozenset([1, 2, 3]), frozenset([2, 3, 4]),
                       frozenset([3, 4, 5, 6])])
        cgc = nx.chordal_graph_cliques
        assert cgc(self.connected_chordal_G) == cliques

    def test_complete_to_chordal_graph(self):
        fgrg = nx.fast_gnp_random_graph
        test_graphs = [nx.barbell_graph(6, 2), nx.cycle_graph(15),
                       nx.wheel_graph(20), nx.grid_graph([10, 4]),
                       nx.ladder_graph(15), nx.star_graph(5),
                       nx.bull_graph(), fgrg(20, 0.3, seed=1)]
        for G in test_graphs:
            H, a = nx.complete_to_chordal_graph(G)
            assert nx.is_chordal(H)
            assert len(a) == H.number_of_nodes()
            if nx.is_chordal(G):
                assert G.number_of_edges() == H.number_of_edges()
                assert set(a.values()) == {0}
            else:
                assert len(set(a.values())) == H.number_of_nodes()



# -*- encoding: utf-8 -*-
# test_random_graphs.py - unit tests for random graph generators
#
# Copyright 2010-2019 NetworkX developers.
#
# This file is part of NetworkX.
#
# NetworkX is distributed under a BSD license; see LICENSE.txt for more
# information.
"""Unit tests for the :mod:`networkx.generators.random_graphs` module.

"""
import pytest

from networkx.exception import NetworkXError
from networkx.generators.random_graphs import barabasi_albert_graph
from networkx.generators.random_graphs import dual_barabasi_albert_graph
from networkx.generators.random_graphs import extended_barabasi_albert_graph
from networkx.generators.random_graphs import binomial_graph
from networkx.generators.random_graphs import connected_watts_strogatz_graph
from networkx.generators.random_graphs import dense_gnm_random_graph
from networkx.generators.random_graphs import erdos_renyi_graph
from networkx.generators.random_graphs import fast_gnp_random_graph
from networkx.generators.random_graphs import gnm_random_graph
from networkx.generators.random_graphs import gnp_random_graph
from networkx.generators.random_graphs import newman_watts_strogatz_graph
from networkx.generators.random_graphs import powerlaw_cluster_graph
from networkx.generators.random_graphs import random_kernel_graph
from networkx.generators.random_graphs import random_lobster
from networkx.generators.random_graphs import random_powerlaw_tree
from networkx.generators.random_graphs import random_powerlaw_tree_sequence
from networkx.generators.random_graphs import random_regular_graph
from networkx.generators.random_graphs import random_shell_graph
from networkx.generators.random_graphs import watts_strogatz_graph


class TestGeneratorsRandom(object):

    def smoke_test_random_graph(self):
        seed = 42
        G = gnp_random_graph(100, 0.25, seed)
        G = gnp_random_graph(100, 0.25, seed, directed=True)
        G = binomial_graph(100, 0.25, seed)
        G = erdos_renyi_graph(100, 0.25, seed)
        G = fast_gnp_random_graph(100, 0.25, seed)
        G = fast_gnp_random_graph(100, 0.25, seed, directed=True)
        G = gnm_random_graph(100, 20, seed)
        G = gnm_random_graph(100, 20, seed, directed=True)
        G = dense_gnm_random_graph(100, 20, seed)

        G = watts_strogatz_graph(10, 2, 0.25, seed)
        assert len(G) == 10
        assert G.number_of_edges() == 10

        G = connected_watts_strogatz_graph(10, 2, 0.1, tries=10, seed=seed)
        assert len(G) == 10
        assert G.number_of_edges() == 10
        pytest.raises(NetworkXError, connected_watts_strogatz_graph, \
                      10, 2, 0.1, tries=0)

        G = watts_strogatz_graph(10, 4, 0.25, seed)
        assert len(G) == 10
        assert G.number_of_edges() == 20

        G = newman_watts_strogatz_graph(10, 2, 0.0, seed)
        assert len(G) == 10
        assert G.number_of_edges() == 10

        G = newman_watts_strogatz_graph(10, 4, 0.25, seed)
        assert len(G) == 10
        assert G.number_of_edges() >= 20

        G = barabasi_albert_graph(100, 1, seed)
        G = barabasi_albert_graph(100, 3, seed)
        assert G.number_of_edges() == (97 * 3)

        G = extended_barabasi_albert_graph(100, 1, 0, 0, seed)
        assert G.number_of_edges() == 99
        G = extended_barabasi_albert_graph(100, 3, 0, 0, seed)
        assert G.number_of_edges() == 97 * 3
        G = extended_barabasi_albert_graph(100, 1, 0, 0.5, seed)
        assert G.number_of_edges() == 99
        G = extended_barabasi_albert_graph(100, 2, 0.5, 0, seed)
        assert G.number_of_edges() > 100 * 3
        assert G.number_of_edges() < 100 * 4

        G = extended_barabasi_albert_graph(100, 2, 0.3, 0.3, seed)
        assert G.number_of_edges() > 100 * 2
        assert G.number_of_edges() < 100 * 4

        G = powerlaw_cluster_graph(100, 1, 1.0, seed)
        G = powerlaw_cluster_graph(100, 3, 0.0, seed)
        assert G.number_of_edges() == (97 * 3)

        G = random_regular_graph(10, 20, seed)

        pytest.raises(NetworkXError, random_regular_graph, 3, 21)
        pytest.raises(NetworkXError, random_regular_graph, 33, 21)

        constructor = [(10, 20, 0.8), (20, 40, 0.8)]
        G = random_shell_graph(constructor, seed)

        G = random_lobster(10, 0.1, 0.5, seed)

        # difficult to find seed that requires few tries
        seq = random_powerlaw_tree_sequence(10, 3, seed=14, tries=1)
        G = random_powerlaw_tree(10, 3, seed=14, tries=1)

    def test_dual_barabasi_albert(self, m1=1, m2=4, p=0.5):
        """
        Tests that the dual BA random graph generated behaves consistently.

        Tests the exceptions are raised as expected.

        The graphs generation are repeated several times to prevent lucky shots

        """
        seed = 42
        repeats = 2

        while repeats:
            repeats -= 1

            # This should be BA with m = m1
            BA1 = barabasi_albert_graph(100, m1, seed)
            DBA1 = dual_barabasi_albert_graph(100, m1, m2, 1, seed)
            assert BA1.size() == DBA1.size()

            # This should be BA with m = m2
            BA2 = barabasi_albert_graph(100, m2, seed)
            DBA2 = dual_barabasi_albert_graph(100, m1, m2, 0, seed)
            assert BA2.size() == DBA2.size()

        # Testing exceptions
        dbag = dual_barabasi_albert_graph
        pytest.raises(NetworkXError, dbag, m1, m1, m2, 0)
        pytest.raises(NetworkXError, dbag, m2, m1, m2, 0)
        pytest.raises(NetworkXError, dbag, 100, m1, m2, -0.5)
        pytest.raises(NetworkXError, dbag, 100, m1, m2, 1.5)

    def test_extended_barabasi_albert(self, m=2):
        """
        Tests that the extended BA random graph generated behaves consistently.

        Tests the exceptions are raised as expected.

        The graphs generation are repeated several times to prevent lucky-shots

        """
        seed = 42
        repeats = 2
        BA_model = barabasi_albert_graph(100, m, seed)
        BA_model_edges = BA_model.number_of_edges()

        while repeats:
            repeats -= 1

            # This behaves just like BA, the number of edges must be the same
            G1 = extended_barabasi_albert_graph(100, m, 0, 0, seed)
            assert G1.size() == BA_model_edges

            # More than twice more edges should have been added
            G1 = extended_barabasi_albert_graph(100, m, 0.8, 0, seed)
            assert G1.size() > BA_model_edges * 2

            # Only edge rewiring, so the number of edges less than original
            G2 = extended_barabasi_albert_graph(100, m, 0, 0.8, seed)
            assert G2.size() == BA_model_edges

            # Mixed scenario: less edges than G1 and more edges than G2
            G3 = extended_barabasi_albert_graph(100, m, 0.3, 0.3, seed)
            assert G3.size() > G2.size()
            assert G3.size() < G1.size()

        # Testing exceptions
        ebag = extended_barabasi_albert_graph
        pytest.raises(NetworkXError, ebag, m, m, 0, 0)
        pytest.raises(NetworkXError, ebag, 1, 0.5, 0, 0)
        pytest.raises(NetworkXError, ebag, 100, 2, 0.5, 0.5)

    def test_random_zero_regular_graph(self):
        """Tests that a 0-regular graph has the correct number of nodes and
        edges.

        """
        seed = 42
        G = random_regular_graph(0, 10, seed)
        assert len(G) == 10
        assert sum(1 for _ in G.edges()) == 0

    def test_gnp(self):
        for generator in [gnp_random_graph, binomial_graph, erdos_renyi_graph,
                          fast_gnp_random_graph]:
            G = generator(10, -1.1)
            assert len(G) == 10
            assert sum(1 for _ in G.edges()) == 0

            G = generator(10, 0.1)
            assert len(G) == 10

            G = generator(10, 0.1, seed=42)
            assert len(G) == 10

            G = generator(10, 1.1)
            assert len(G) == 10
            assert sum(1 for _ in G.edges()) == 45

            G = generator(10, -1.1, directed=True)
            assert G.is_directed()
            assert len(G) == 10
            assert sum(1 for _ in G.edges()) == 0

            G = generator(10, 0.1, directed=True)
            assert G.is_directed()
            assert len(G) == 10

            G = generator(10, 1.1, directed=True)
            assert G.is_directed()
            assert len(G) == 10
            assert sum(1 for _ in G.edges()) == 90

            # assert that random graphs generate all edges for p close to 1
            edges = 0
            runs = 100
            for i in range(runs):
                edges += sum(1 for _ in generator(10, 0.99999, directed=True).edges())
            assert abs(edges / float(runs) - 90) <= runs * 2.0 / 100

    def test_gnm(self):
        G = gnm_random_graph(10, 3)
        assert len(G) == 10
        assert sum(1 for _ in G.edges()) == 3

        G = gnm_random_graph(10, 3, seed=42)
        assert len(G) == 10
        assert sum(1 for _ in G.edges()) == 3

        G = gnm_random_graph(10, 100)
        assert len(G) == 10
        assert sum(1 for _ in G.edges()) == 45

        G = gnm_random_graph(10, 100, directed=True)
        assert len(G) == 10
        assert sum(1 for _ in G.edges()) == 90

        G = gnm_random_graph(10, -1.1)
        assert len(G) == 10
        assert sum(1 for _ in G.edges()) == 0

    def test_watts_strogatz_big_k(self):
        #Test to make sure than n <= k
        pytest.raises(NetworkXError, watts_strogatz_graph, 10, 11, 0.25)
        pytest.raises(NetworkXError, newman_watts_strogatz_graph, 10, 11, 0.25)
        
        # could create an infinite loop, now doesn't
        # infinite loop used to occur when a node has degree n-1 and needs to rewire
        watts_strogatz_graph(10, 9, 0.25, seed=0)
        newman_watts_strogatz_graph(10, 9, 0.5, seed=0)

        #Test k==n scenario
        watts_strogatz_graph(10, 10, 0.25, seed=0)
        newman_watts_strogatz_graph(10, 10, 0.25, seed=0)

    def test_random_kernel_graph(self):
        def integral(u, w, z):
            return c * (z - w)

        def root(u, w, r):
            return r / c + w
        c = 1
        graph = random_kernel_graph(1000, integral, root)
        graph = random_kernel_graph(1000, integral, root, seed=42)
        assert len(graph) == 1000
